@@ -24,10 +24,33 @@ package org.klomp.snark;
 /**
  * Callback used when some peer changes state.
  */
-public interface CoordinatorListener
+interface CoordinatorListener
 {
   /**
    * Called when the PeerCoordinator notices a change in the state of a peer.
    */
   void peerChange(PeerCoordinator coordinator, Peer peer);
+
+  /**
+   * Called when the PeerCoordinator got the MetaInfo via magnet.
+   * @since 0.8.4
+   */
+  void gotMetaInfo(PeerCoordinator coordinator, MetaInfo metainfo);
+
+  /**
+   * Is this number of uploaders over the per-torrent limit?
+   */
+  public boolean overUploadLimit(int uploaders);
+
+  /**
+   * Are we currently over the upstream bandwidth limit?
+   */
+  public boolean overUpBWLimit();
+
+  /**
+   * Is the total (in Bps) over the upstream bandwidth limit?
+   */
+  public boolean overUpBWLimit(long total);
+
+  public void addMessage(String message);
 }
