@@ -51,6 +51,7 @@ import net.i2p.util.SystemVersion;
 public class TunnelController implements Logging {
     private final Log _log;
     private Properties _config;
+    private File _configFile;
     private final I2PTunnel _tunnel;
     private final List<String> _messages;
     private List<I2PSession> _sessions;
@@ -86,6 +87,8 @@ public class TunnelController implements Logging {
     public static final String PROP_TARGET_PORT = "targetPort";
     public static final String PROP_TYPE = "type";
     public static final String PROP_FILTER = "filterDefinition";
+    /** @since 0.9.42 */
+    public static final String PROP_CONFIG_FILE = "configFile";
 
     /**
      * all of these are @since 0.9.33 (moved from TunnelConfig)
@@ -174,9 +177,12 @@ public class TunnelController implements Logging {
      * The config may contain a large number of options - only ones that begin in
      * the prefix should be used (and, in turn, that prefix should be stripped off
      * before being interpreted by this controller)
-     * 
+     *
+     * If config contains the "configFile" property, it will be set as the config path
+     * and may be retrieved with getConfigFile().
+     *
      * Defaults in config properties are not recommended, they may or may not be honored.
-     * 
+     *
      * @param config original key=value mapping non-null
      * @param prefix beginning of key values that are relevant to this tunnel
      */
@@ -185,8 +191,16 @@ public class TunnelController implements Logging {
     }
 
     /**
+     * Create a new controller for a tunnel out of the specific config options.
+     * The config may contain a large number of options - only ones that begin in
+     * the prefix should be used (and, in turn, that prefix should be stripped off
+     * before being interpreted by this controller)
+     *
+     * If config contains the "configFile" property, it will be set as the config path
+     * and may be retrieved with getConfigFile().
+     *
      * Defaults in config properties are not recommended, they may or may not be honored.
-     * 
+     *
      * @param config original key=value mapping non-null
      * @param prefix beginning of key values that are relevant to this tunnel
      * @param createKey for servers, whether we want to create a brand new destination
@@ -219,7 +233,7 @@ public class TunnelController implements Logging {
             log("No filename specified for the private key");
             return false;
         }
-        
+
         if (keyFile.exists()) {
             //log("Not overwriting existing private keys in " + keyFile.getAbsolutePath());
             return true;
@@ -324,7 +338,7 @@ public class TunnelController implements Logging {
             } else if (len > 128) {
                 // copy of excess data handled in KeyCertificate constructor
             }
-        
+
             out = new SecureFileOutputStream(altFile);
             d.writeBytes(out);
             priv.writeBytes(out);
@@ -411,7 +425,7 @@ public class TunnelController implements Logging {
                 return;
         }
 
-        String type = getType(); 
+        String type = getType();
         if ( (type == null) || (type.length() <= 0) ) {
             changeState(TunnelState.STOPPED);
             if (_log.shouldLog(Log.ERROR))
@@ -496,7 +510,7 @@ public class TunnelController implements Logging {
         String dest = getTargetDestination();
         String sharedClient = getSharedClient();
         if (getPersistentClientKey()) {
-            String privKeyFile = getPrivKeyFile(); 
+            String privKeyFile = getPrivKeyFile();
             _tunnel.runIrcClient(new String[] { listenPort, dest, sharedClient, privKeyFile }, this);
         } else {
             _tunnel.runIrcClient(new String[] { listenPort, dest, sharedClient }, this);
@@ -515,7 +529,7 @@ public class TunnelController implements Logging {
                 props.setProperty(I2PSOCKSTunnel.PROP_PROXY_DEFAULT, proxyList);
         }
         if (getPersistentClientKey()) {
-            String privKeyFile = getPrivKeyFile(); 
+            String privKeyFile = getPrivKeyFile();
             _tunnel.runSOCKSTunnel(new String[] { listenPort, "false", privKeyFile }, this);
         } else {
             _tunnel.runSOCKSTunnel(new String[] { listenPort, sharedClient }, this);
@@ -535,7 +549,7 @@ public class TunnelController implements Logging {
                 props.setProperty(I2PSOCKSTunnel.PROP_PROXY_DEFAULT, proxyList);
         }
         if (getPersistentClientKey()) {
-            String privKeyFile = getPrivKeyFile(); 
+            String privKeyFile = getPrivKeyFile();
             _tunnel.runSOCKSIRCTunnel(new String[] { listenPort, "false", privKeyFile }, this);
         } else {
             _tunnel.runSOCKSIRCTunnel(new String[] { listenPort, sharedClient }, this);
@@ -561,11 +575,11 @@ public class TunnelController implements Logging {
             _tunnel.runListenOn(new String[] { listenOn }, this);
         }
         String listenPort = getTargetPort();
-        String privKeyFile = getPrivKeyFile(); 
+        String privKeyFile = getPrivKeyFile();
         _tunnel.runStreamrServer(new String[] { listenPort, privKeyFile }, this);
     }
 
-    /** 
+    /**
      * Note the fact that we are using some sessions, so that they dont get
      * closed by some other tunnels
      */
@@ -587,7 +601,7 @@ public class TunnelController implements Logging {
         }
     }
 
-    /** 
+    /**
      * Note the fact that we are no longer using some sessions, and if
      * no other tunnels are using them, close them.
      */
@@ -607,7 +621,7 @@ public class TunnelController implements Logging {
         }
     }
 
-    /** 
+    /**
      *  Get all the sessions we may be using.
      *
      *  @return a copy, non-null
@@ -625,11 +639,11 @@ public class TunnelController implements Logging {
 
     private void startClient() {
         setListenOn();
-        String listenPort = getListenPort(); 
+        String listenPort = getListenPort();
         String dest = getTargetDestination();
         String sharedClient = getSharedClient();
         if (getPersistentClientKey()) {
-            String privKeyFile = getPrivKeyFile(); 
+            String privKeyFile = getPrivKeyFile();
             _tunnel.runClient(new String[] { listenPort, dest, sharedClient, privKeyFile }, this);
         } else {
             _tunnel.runClient(new String[] { listenPort, dest, sharedClient }, this);
@@ -637,17 +651,17 @@ public class TunnelController implements Logging {
     }
 
     private void startServer() {
-        String targetHost = getTargetHost(); 
-        String targetPort = getTargetPort(); 
-        String privKeyFile = getPrivKeyFile(); 
+        String targetHost = getTargetHost();
+        String targetPort = getTargetPort();
+        String privKeyFile = getPrivKeyFile();
         _tunnel.runServer(new String[] { targetHost, targetPort, privKeyFile }, this);
     }
 
     private void startHttpServer() {
-        String targetHost = getTargetHost(); 
-        String targetPort = getTargetPort(); 
-        String spoofedHost = getSpoofedHost(); 
-        String privKeyFile = getPrivKeyFile(); 
+        String targetHost = getTargetHost();
+        String targetPort = getTargetPort();
+        String spoofedHost = getSpoofedHost();
+        String privKeyFile = getPrivKeyFile();
         _tunnel.runHttpServer(new String[] { targetHost, targetPort, spoofedHost, privKeyFile }, this);
     }
 
@@ -662,9 +676,9 @@ public class TunnelController implements Logging {
     }
 
     private void startIrcServer() {
-        String targetHost = getTargetHost(); 
-        String targetPort = getTargetPort(); 
-        String privKeyFile = getPrivKeyFile(); 
+        String targetHost = getTargetHost();
+        String targetPort = getTargetPort();
+        String privKeyFile = getPrivKeyFile();
         _tunnel.runIrcServer(new String[] { targetHost, targetPort, privKeyFile }, this);
     }
 
@@ -723,10 +737,10 @@ public class TunnelController implements Logging {
 
     private void setI2CPOptions() {
         String host = getI2CPHost();
-        if ( (host != null) && (host.length() > 0) ) 
+        if ( (host != null) && (host.length() > 0) )
             _tunnel.host = host;
         // woohah, special casing for people with ipv6/etc
-        if ("localhost".equals(_tunnel.host)) 
+        if ("localhost".equals(_tunnel.host))
             _tunnel.host = "127.0.0.1";
         String port = getI2CPPort();
         if ( (port != null) && (port.length() > 0) ) {
@@ -789,18 +803,28 @@ public class TunnelController implements Logging {
      *  As of 0.9.1, updates the options on an existing session
      */
     public void setConfig(Properties config, String prefix) {
-        Properties props = new Properties();
-        for (Map.Entry<Object, Object> e : config.entrySet()) {
-            String key = (String) e.getKey();
-            if (key.startsWith(prefix)) {
-                key = key.substring(prefix.length());
-                String val = (String) e.getValue();
-                props.setProperty(key, val);
+        Properties props;
+        if (prefix.length() > 0) {
+            props = new Properties();
+            for (Map.Entry<Object, Object> e : config.entrySet()) {
+                String key = (String) e.getKey();
+                if (key.startsWith(prefix)) {
+                    key = key.substring(prefix.length());
+                    String val = (String) e.getValue();
+                    props.setProperty(key, val);
+                }
             }
+        } else {
+            props = config;
         }
         Properties oldConfig = _config;
         _config = props;
 
+        // save the config file this was loaded from, if passed in
+        String cname = _config.getProperty(PROP_CONFIG_FILE);
+        if (cname != null && _configFile == null)
+            _configFile = new File(cname);
+
         // Set up some per-type defaults
         // This really isn't the best spot to do this but for servers in particular,
         // it's hard to override settings in the subclass since the session connect
@@ -836,7 +860,7 @@ public class TunnelController implements Logging {
             }
             if (!isClient(type)) {
                 _tunnel.filterDefinition = _config.getProperty(PROP_FILTER);
-              
+
                 String p1 = _config.getProperty(OPT_MAX_CONNS_MIN, "0");
                 String p2 = _config.getProperty(OPT_MAX_CONNS_HOUR, "0");
                 String p3 = _config.getProperty(OPT_MAX_CONNS_DAY, "0");
@@ -919,21 +943,39 @@ public class TunnelController implements Logging {
     /**
      *  @return a copy
      */
-    public Properties getConfig(String prefix) { 
+    public Properties getConfig(String prefix) {
         Properties rv = new Properties();
-        for (Map.Entry<Object, Object> e : _config.entrySet()) {
-            String key = (String) e.getKey();
-            String val = (String) e.getValue();
-            rv.setProperty(prefix + key, val);
+        if (prefix.length() > 0) {
+            for (Map.Entry<Object, Object> e : _config.entrySet()) {
+                String key = (String) e.getKey();
+                String val = (String) e.getValue();
+                rv.setProperty(prefix + key, val);
+            }
+        } else {
+            rv.putAll(_config);
         }
         return rv;
     }
 
+    /**
+     *  @return the config file as passed into constructor via "configFile" property,
+     *          or as set later, or null
+     *  @since 0.9.42
+     */
+    public File getConfigFile() { return _configFile; }
+
+    /**
+     *  Set the config file. Only do this if previously null.
+     *  @since 0.9.42
+     */
+    public void setConfigFile(File file) { _configFile = file; }
+
     public String getType() { return _config.getProperty(PROP_TYPE); }
     public String getName() { return _config.getProperty(PROP_NAME); }
     public String getDescription() { return _config.getProperty(PROP_DESCR); }
     public String getI2CPHost() { return _config.getProperty(PROP_I2CP_HOST); }
     public String getI2CPPort() { return _config.getProperty(PROP_I2CP_PORT); }
+
     /**
      *  Absolute path to filter definition file
      *  @since 0.9.40
@@ -959,7 +1001,7 @@ public class TunnelController implements Logging {
      *  @since 0.9.17 moved from IndexBean
      */
     public static boolean isClient(String type) {
-        return TYPE_STD_CLIENT.equals(type) || 
+        return TYPE_STD_CLIENT.equals(type) ||
                TYPE_HTTP_CLIENT.equals(type) ||
                TYPE_SOCKS.equals(type) ||
                TYPE_SOCKS_IRC.equals(type) ||
@@ -1150,7 +1192,7 @@ public class TunnelController implements Logging {
             buf.append("<i>").append(description).append("</i><br />\n");
         buf.append("HTTP proxy listening on port ").append(getListenPort());
         String listenOn = getListenOnInterface();
-        if ("0.0.0.0".equals(listenOn)) 
+        if ("0.0.0.0".equals(listenOn))
             buf.append(" (reachable by any machine)");
         else if ("127.0.0.1".equals(listenOn))
             buf.append(" (reachable locally only)");
@@ -1172,7 +1214,7 @@ public class TunnelController implements Logging {
         buf.append("Client tunnel listening on port ").append(getListenPort());
         buf.append(" pointing at ").append(getTargetDestination());
         String listenOn = getListenOnInterface();
-        if ("0.0.0.0".equals(listenOn)) 
+        if ("0.0.0.0".equals(listenOn))
             buf.append(" (reachable by any machine)");
         else if ("127.0.0.1".equals(listenOn))
             buf.append(" (reachable locally only)");
@@ -1251,11 +1293,11 @@ public class TunnelController implements Logging {
     }
 
     /**
-     * Pull off any messages that the I2PTunnel has produced 
+     * Pull off any messages that the I2PTunnel has produced
      *
      * @return list of messages pulled off (each is a String, earliest first)
      */
-    public List<String> clearMessages() { 
+    public List<String> clearMessages() {
         List<String> rv;
         synchronized (_messages) {
             rv = new ArrayList<String>(_messages);
@@ -1268,7 +1310,7 @@ public class TunnelController implements Logging {
      * @since 0.9.15
      */
     @Override
-    public String toString() { 
+    public String toString() {
         return "TC " + getType() + ' ' + getName() + " for " + _tunnel + ' ' + _state;
     }
 }
