@@ -55,10 +55,16 @@ public class GeneralHelper {
     private final I2PAppContext _context;
     protected final TunnelControllerGroup _group;
 
+    /**
+     *  @param tcg may be null ???
+     */
     public GeneralHelper(TunnelControllerGroup tcg) {
         this(I2PAppContext.getGlobalContext(), tcg);
     }
 
+    /**
+     *  @param tcg may be null ???
+     */
     public GeneralHelper(I2PAppContext context, TunnelControllerGroup tcg) {
         _context = context;
         _group = tcg;
@@ -67,12 +73,17 @@ public class GeneralHelper {
     public TunnelController getController(int tunnel) {
         return getController(_group, tunnel);
     }
+
+    /**
+     *  @param tcg may be null
+     *  @return null if not found or tcg is null
+     */
     public static TunnelController getController(TunnelControllerGroup tcg, int tunnel) {
         if (tunnel < 0) return null;
         if (tcg == null) return null;
         List<TunnelController> controllers = tcg.getControllers();
         if (controllers.size() > tunnel)
-            return controllers.get(tunnel); 
+            return controllers.get(tunnel);
         else
             return null;
     }
@@ -80,10 +91,11 @@ public class GeneralHelper {
     public List<String> saveTunnel(int tunnel, TunnelConfig config) {
         return saveTunnel(_context, _group, tunnel, config);
     }
+
     public static List<String> saveTunnel(
             I2PAppContext context, TunnelControllerGroup tcg, int tunnel, TunnelConfig config) {
         List<String> msgs = updateTunnelConfig(tcg, tunnel, config);
-        msgs.addAll(saveConfig(context, tcg));
+        msgs.addAll(saveConfig(context, tcg, tunnel));
         return msgs;
     }
 
@@ -129,21 +141,33 @@ public class GeneralHelper {
                         // config now contains new keystore props
                         String name = props.getProperty(TunnelController.PROP_NAME, "");
                         msgs.add("Created new self-signed certificate for tunnel " + name);
-                    }        
-                } catch (IOException ioe) {       
+                    }
+                } catch (IOException ioe) {
                     msgs.add("Failed to create new self-signed certificate for tunnel " +
                             getTunnelName(tcg, tunnel) + ", check logs: " + ioe);
-                }        
-            }        
-        }        
+                }
+            }
+        }
         if (cur == null) {
             // creating new
             cur = new TunnelController(props, "", true);
             tcg.addController(cur);
             if (cur.getStartOnLoad())
                 cur.startTunnelBackground();
+            try {
+                tcg.saveConfig(cur);
+            } catch (IOException ioe) {
+                msgs.add("Failed to save initial tunnel config after creation " +
+                    cur.getName() + ", check logs:" + ioe);
+            }
         } else {
             cur.setConfig(props, "");
+            try {
+                tcg.saveConfig(cur);
+            } catch (IOException ioe) {
+                msgs.add("Failed to save initial tunnel config after creation " +
+                    cur.getName() + ", check logs:" + ioe);
+            }
         }
         // Only modify other shared tunnels
         // if the current tunnel is shared, and of supported type
@@ -166,6 +190,12 @@ public class GeneralHelper {
                     cOpt.setProperty("option.outbound.nickname", TunnelConfig.SHARED_CLIENT_NICKNAME);
 
                     c.setConfig(cOpt, "");
+                    try {
+                        tcg.saveConfig(c);
+                    } catch (IOException ioe) {
+                        msgs.add("Failed to save initial tunnel config after creation " +
+                            cur.getName() + ", check logs:" + ioe);
+                    }
                 }
             }
         }
@@ -173,10 +203,17 @@ public class GeneralHelper {
         return msgs;
     }
 
-    protected static List<String> saveConfig(I2PAppContext context, TunnelControllerGroup tcg) { 
+    protected static List<String> saveConfig(
+            I2PAppContext context, TunnelControllerGroup tcg, int tunnel) {
         List<String> rv = tcg.clearAllMessages();
         try {
-            tcg.saveConfig();
+            TunnelController cur = getController(tcg, tunnel);
+            if (cur == null) {
+                //List<String> msgs = new ArrayList<String>();
+                rv.add("Invalid tunnel number");
+                return rv;
+            }
+            tcg.saveConfig(cur);
             rv.add(0, _t("Configuration changes saved", context));
         } catch (IOException ioe) {
             Log log = context.logManager().getLog(GeneralHelper.class);
@@ -207,7 +244,11 @@ public class GeneralHelper {
         }
 
         msgs = tcg.removeController(cur);
-        msgs.addAll(saveConfig(context, tcg));
+        try {
+            tcg.removeConfig(cur);
+        }catch (IOException ioe){
+            msgs.add(ioe.toString());
+        }
 
         // Rename private key file if it was a default name in
         // the default directory, so it doesn't get reused when a new
@@ -558,7 +599,7 @@ public class GeneralHelper {
     public boolean getEncrypt(int tunnel) {
         return getBooleanProperty(tunnel, "i2cp.encryptLeaseSet");
     }
-    
+
     /**
      *  @since 0.9.40
      */
@@ -592,7 +633,7 @@ public class GeneralHelper {
         }
         return 0;
     }
-    
+
     /**
      *  @since 0.9.40
      */
@@ -665,6 +706,21 @@ public class GeneralHelper {
         return type.getCode();
     }
 
+    /**
+     *  @param encType code
+     *  @since 0.9.44
+     */
+    public boolean hasEncType(int tunnel, int encType) {
+        String senc = getProperty(tunnel, "i2cp.leaseSetEncType", "0");
+        String[] senca = DataHelper.split(senc, ",");
+        String se = Integer.toString(encType);
+        for (int i = 0; i < senca.length; i++) {
+            if (se.equals(senca[i]))
+                return true;
+        }
+        return false;
+    }
+
     /**
      *  Random keys
      */
@@ -703,7 +759,7 @@ public class GeneralHelper {
             return 2;
         return 0;
     }
-    
+
     public String getAccessList(int tunnel) {
         return getProperty(tunnel, "i2cp.accessList", "").replace(",", "\n");
     }
@@ -720,12 +776,12 @@ public class GeneralHelper {
         }
         return "";
     }
-    
+
     public String getJumpList(int tunnel) {
         return getProperty(tunnel, I2PTunnelHTTPClient.PROP_JUMP_SERVERS,
                            I2PTunnelHTTPClient.DEFAULT_JUMP_SERVERS).replace(",", "\n");
     }
-    
+
     public boolean getCloseOnIdle(int tunnel, boolean def) {
         return getBooleanProperty(tunnel, "i2cp.closeOnIdle", def);
     }
@@ -776,15 +832,15 @@ public class GeneralHelper {
     public String getProxyAuth(int tunnel) {
         return getProperty(tunnel, I2PTunnelHTTPClientBase.PROP_AUTH, "false");
     }
-    
+
     public boolean getOutproxyAuth(int tunnel) {
         return getBooleanProperty(tunnel, I2PTunnelHTTPClientBase.PROP_OUTPROXY_AUTH);
     }
-    
+
     public String getOutproxyUsername(int tunnel) {
         return getProperty(tunnel, I2PTunnelHTTPClientBase.PROP_OUTPROXY_USER, "");
     }
-    
+
     public String getOutproxyPassword(int tunnel) {
         if (getOutproxyUsername(tunnel).length() <= 0)
             return "";
@@ -908,7 +964,13 @@ public class GeneralHelper {
                     buf.append(' ');
                 else
                     space = true;
-                buf.append(e.getKey()).append('=').append(e.getValue());
+                buf.append(e.getKey()).append('=');
+                String v = e.getValue();
+                if (v.contains(" ") || v.contains("\t")) {
+                    buf.append('"').append(v).append('"');
+                } else {
+                    buf.append(v);
+                }
             }
             return DataHelper.escapeHTML(buf.toString());
         } else {
