@@ -1239,6 +1239,14 @@ public class IndexBean {
         // Otherwise this only works on a new tunnel...
     }
 
+    /**
+     * Adds to existing, comma separated
+     * @since 0.9.44
+     */
+    public void setEncType(String s) {
+        _config.setEncType(s);
+    }
+
     /**
      *  Random keys, hidden in forms
      *  @since 0.9.18
