@@ -7,6 +7,7 @@ import java.io.IOException;
 import java.io.OutputStream;
 import java.net.URI;
 import java.net.URISyntaxException;
+import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Locale;
@@ -244,12 +245,33 @@ public class ConfigClientsHandler extends FormHandler {
     }
     
     private void saveClientChanges() {
+        try {
+            synchronized(ClientAppConfig.class) {
+                saveClientChanges2();
+            }
+            addFormNotice(_t("Client configuration saved successfully"));
+        } catch (IOException ioe) {
+            addFormError(_t("Error saving the configuration (applied but not saved) - please see the error logs"));
+            addFormError(ioe.getLocalizedMessage());
+        }
+    }
+
+    private void saveClientChanges2() throws IOException {
+        // will save if not split config
         List<ClientAppConfig> clients = ClientAppConfig.getClientApps(_context);
+        // will save if split config
+        List<ClientAppConfig> saveClients = new ArrayList<ClientAppConfig>(clients.size() + 1);
+        boolean isSplitConfig = ClientAppConfig.isSplitConfig(_context);
         for (int cur = 0; cur < clients.size(); cur++) {
             ClientAppConfig ca = clients.get(cur);
             Object val = _settings.get(cur + ".enabled");
-            if (! (RouterConsoleRunner.class.getName().equals(ca.className)))
-                ca.disabled = val == null;
+            if (! (RouterConsoleRunner.class.getName().equals(ca.className))) {
+                boolean newval = val == null;
+                if (ca.disabled != newval) {
+                    ca.disabled = newval;
+                    saveClients.add(ca);
+                }
+            }
             // edit of an existing entry
             if (_context.getBooleanProperty(ConfigClientsHelper.PROP_ENABLE_CLIENT_CHANGE) ||
                 isAdvanced()) {
@@ -265,6 +287,7 @@ public class ConfigClientsHandler extends FormHandler {
                     ca.className = clss;
                     ca.args = args;
                     ca.clientName = getJettyString("nofilter_name" + cur);
+                    saveClients.add(ca);
                 }
             }
         }
@@ -288,13 +311,20 @@ public class ConfigClientsHandler extends FormHandler {
                 ClientAppConfig ca = new ClientAppConfig(clss, name, args, 2*60*1000,
                                                          _settings.get(newClient + ".enabled") == null);  // true for disabled
                 clients.add(ca);
+                saveClients.add(ca);
                 addFormNotice(_t("New client added") + ": " + name + " (" + clss + ").");
             }
         }
-
-        ClientAppConfig.writeClientAppConfig(_context, clients);
-        addFormNotice(_t("Client configuration saved successfully"));
-        //addFormNotice(_t("Restart required to take effect"));
+        if (isSplitConfig) {
+            // Only save what changed
+            // TODO this will lose other clients in the same file
+            for (ClientAppConfig ca : saveClients) {
+                ClientAppConfig.writeClientAppConfig(_context, ca);
+            }
+        } else {
+            // Save all
+            ClientAppConfig.writeClientAppConfig(_context, clients);
+        }
     }
 
     /**
@@ -347,9 +377,14 @@ public class ConfigClientsHandler extends FormHandler {
             addFormError(_t("Bad client index."));
             return;
         }
-        ClientAppConfig ca = clients.remove(i);
-        ClientAppConfig.writeClientAppConfig(_context, clients);
-        addFormNotice(_t("Client {0} deleted", ca.clientName));
+        ClientAppConfig ca = clients.get(i);
+        try {
+            ClientAppConfig.deleteClientAppConfig(ca);
+            addFormNotice(_t("Client {0} deleted", ca.clientName));
+        } catch (IOException ioe) {
+            addFormError(_t("Error saving the configuration (applied but not saved) - please see the error logs"));
+            addFormError(ioe.getLocalizedMessage());
+        }
     }
 
     private void saveWebAppChanges() {
