@@ -398,7 +398,7 @@ public class GraphHelper extends FormHandler {
                 _out.write(Integer.toString(times[i]));
                 _out.write('"');
                 if (times[i] == _refreshDelaySeconds)
-                    _out.write(" selected=\"selected\"");
+                    _out.write(HelperBase.SELECTED);
                 _out.write('>');
                 if (times[i] > 0)
                     _out.write(DataHelper.formatDuration2(times[i] * 1000));
