@@ -150,6 +150,22 @@ public class PeerHelper extends HelperBase {
             out.write(buf.toString());
         } else if ("upnp".equals(_transport)) {
             // UPnP Status
+            StringBuilder buf = new StringBuilder(512);
+            buf.append("<h3>").append(_t("UPnP Overview"))
+               .append("</h3><div id=\"upnpoverview\"><p>")
+               .append(_t("UPnP is used to communicate with Internet Gateway Devices (IGDs) to detect the external IP address and forward ports."))
+               .append(' ')
+               .append(_t("If UPnP is not working, it may be for one of the following reasons:"))
+               .append("</p><ul id=\"upnphelp\"><li>").append(_t("No UPnP-compatible device present"))
+               .append("</li>\n<li>").append(_t("UPnP disabled on the device"))
+               .append("</li>\n<li>").append(_t("Software firewall interference with UPnP"))
+               .append("</li>\n<li>").append(_t("Bugs in the device's UPnP implementation"))
+               .append("</li>\n<li>").append(_t("Multiple firewall/routers in the internet connection path"))
+               .append("</li>\n<li>").append(_t("UPnP device change, reset, or address change"))
+               .append("</li>\n</ul><p>")
+               .append(_t("UPnP may be enabled or disabled on the Network Configuration page, but a change requires a router restart to take effect."))
+               .append("</p></div>");
+            out.write(buf.toString());
             _context.commSystem().renderStatusHTML(_out, _urlBase, _sortFlags);
         }
         out.flush();
@@ -507,35 +523,37 @@ public class PeerHelper extends HelperBase {
             //    buf.append(' ').append(_context.blocklist().toStr(ip));
             buf.append("</td><td class=\"cells\" nowrap align=\"left\">");
             if (peer.isInbound())
-                buf.append("<img src=\"/themes/console/images/inbound.png\" alt=\"Inbound\" title=\"").append(_t("Inbound")).append("\">");
+                buf.append("<img src=\"/themes/console/images/inbound.png\" alt=\"Inbound\" title=\"").append(_t("Inbound"));
             else
-                buf.append("<img src=\"/themes/console/images/outbound.png\" alt=\"Outbound\" title=\"").append(_t("Outbound")).append("\">");
+                buf.append("<img src=\"/themes/console/images/outbound.png\" alt=\"Outbound\" title=\"").append(_t("Outbound"));
+            buf.append("\">");
             if (peer.getWeRelayToThemAs() > 0)
                 buf.append("&nbsp;&nbsp;<img src=\"/themes/console/images/outbound.png\" height=\"8\" width=\"12\" alt=\"^\" title=\"").append(_t("We offered to introduce them")).append("\">");
             if (peer.getTheyRelayToUsAs() > 0)
                 buf.append("&nbsp;&nbsp;<img src=\"/themes/console/images/inbound.png\" height=\"8\" width=\"12\" alt=\"V\" title=\"").append(_t("They offered to introduce us")).append("\">");
 
-            boolean appended = false;
-            //if (_activeThrottle.isChoked(peer.getRemotePeer())) {
-            //    buf.append("<br><i>").append(_t("Choked")).append("</i>");
-            //    appended = true;
-            //}
-            int cfs = peer.getConsecutiveFailedSends();
-            if (cfs > 0) {
-                if (!appended) buf.append("<br>");
-                buf.append(" <i>");
-                buf.append(ngettext("{0} fail", "{0} fails", cfs));
-                buf.append("</i>");
-                appended = true;
+            if (isAdvanced) {
+                boolean appended = false;
+                //if (_activeThrottle.isChoked(peer.getRemotePeer())) {
+                //    buf.append("<br><i>").append(_t("Choked")).append("</i>");
+                //    appended = true;
+                //}
+                int cfs = peer.getConsecutiveFailedSends();
+                if (cfs > 0) {
+                    if (!appended) buf.append("<br>");
+                    buf.append(" <i>");
+                    buf.append(ngettext("{0} fail", "{0} fails", cfs));
+                    buf.append("</i>");
+                    appended = true;
+                }
+                if (_context.banlist().isBanlisted(peer.getRemotePeer(), "SSU")) {
+                    if (!appended) buf.append("<br>");
+                    buf.append(" <i>").append(_t("Banned")).append("</i>");
+                }
+                //byte[] ip = getIP(peer.getRemotePeer());
+                //if (ip != null)
+                //    buf.append(' ').append(_context.blocklist().toStr(ip));
             }
-            if (_context.banlist().isBanlisted(peer.getRemotePeer(), "SSU")) {
-                if (!appended) buf.append("<br>");
-                buf.append(" <i>").append(_t("Banned")).append("</i>");
-                appended = true;
-            }
-            //byte[] ip = getIP(peer.getRemotePeer());
-            //if (ip != null)
-            //    buf.append(' ').append(_context.blocklist().toStr(ip));
             buf.append("</td>");
 
             buf.append("<td class=\"cells peeripv6\" align=\"center\">");
