@@ -3,9 +3,7 @@ package net.i2p.router.web.helpers;
 import java.io.File;
 import java.io.IOException;
 import java.io.Writer;
-import java.text.DateFormat;
 import java.util.Collections;
-import java.util.Date;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -152,22 +150,10 @@ class SummaryBarRenderer {
 
                    "<table id=\"sb_help\"><tr><td>" +
 
-                   "<a href=\"/help#advancedsettings\" target=\"_top\" title=\"")
-           .append(_t("A guide to some of the less-used configuration settings"))
-           .append("\">")
-           .append(nbsp(_t("Advanced Settings")))
-           .append("</a>\n" +
-
-                   "<a href=\"/help#changelog\" target=\"_top\" title=\"")
+                   "<a href=\"/viewhistory\" target=\"_top\" title=\"")
            .append(_t("Recent development changes to the router"))
            .append("\">")
            .append(nbsp(_t("Changelog")))
-           .append("</a>\n" +
-
-                   "<a href=\"/help#configurationhelp\" target=\"_top\" title=\"")
-           .append(_t("An introduction to configuring your router"))
-           .append("\">")
-           .append(nbsp(_t("Configuration")))
            .append("</a>\n" +
 
                    "<a href=\"/help#faq\" target=\"_top\" title=\"")
@@ -176,7 +162,7 @@ class SummaryBarRenderer {
            .append(nbsp(_t("FAQ")))
            .append("</a>\n" +
 
-                   "<a href=\"/help#legal\" target=\"_top\" title=\"")
+                   "<a href=\"/viewlicense\" target=\"_top\" title=\"")
            .append(_t("Information regarding software and licenses used by I2P"))
            .append("\">")
            .append(nbsp(_t("Legal")))
@@ -245,26 +231,15 @@ class SummaryBarRenderer {
            .append("</a>\n");
         }
 
-        int port = pm.getPort(PortMapper.SVC_EEPSITE);
-        int sslPort = pm.getPort(PortMapper.SVC_HTTPS_EEPSITE);
-        if (sslPort > 0 || port > 0) {
-           String svc;
-           if (sslPort > 0) {
-               buf.append("<a href=\"https://");
-               svc = PortMapper.SVC_HTTPS_EEPSITE;
-               port = sslPort;
-           } else {
-               buf.append("<a href=\"http://");
-               svc = PortMapper.SVC_EEPSITE;
-           }
-           buf.append(pm.getActualHost(svc, "127.0.0.1"))
-           .append(':')
-           .append(port)
-           .append("/\" target=\"_blank\" title=\"")
-           .append(_t("Local web server"))
-           .append("\">")
-           .append(nbsp(_t("Web Server")))
-           .append("</a>\n");
+        String url = getEepsiteURL(pm);
+        if (url != null) {
+            buf.append("<a href=\"")
+               .append(url)
+               .append("\" target=\"_blank\" title=\"")
+               .append(_t("Local web server"))
+               .append("\">")
+               .append(nbsp(_t("Web Server")))
+               .append("</a>\n");
         }
 
         buf.append(NavHelper.getClientAppLinks(_context))
@@ -273,6 +248,37 @@ class SummaryBarRenderer {
         return buf.toString();
     }
 
+    /**
+     *  @return null if none
+     *  @since 0.9.43 split out from above, used by HomeHelper, fixed for IPv6
+     */
+    static String getEepsiteURL(PortMapper pm) {
+        int port = pm.getPort(PortMapper.SVC_EEPSITE);
+        int sslPort = pm.getPort(PortMapper.SVC_HTTPS_EEPSITE);
+        if (port <= 0 && sslPort <= 0)
+            return null;
+        String svc;
+        StringBuilder buf = new StringBuilder(32);
+        if (sslPort > 0) {
+            buf.append("https://");
+            svc = PortMapper.SVC_HTTPS_EEPSITE;
+            port = sslPort;
+        } else {
+            buf.append("http://");
+            svc = PortMapper.SVC_EEPSITE;
+        }
+        String host = pm.getActualHost(svc, "127.0.0.1");
+        if (host.contains(":"))
+            buf.append('[');
+        buf.append(host);
+        if (host.contains(":"))
+            buf.append(']');
+        buf.append(':')
+           .append(port)
+           .append('/');
+        return buf.toString();
+    }
+
     public String renderI2PInternalsHTML() {
         StringBuilder buf = new StringBuilder(512);
         buf.append("<h3><a href=\"/config\" target=\"_top\" title=\"")
@@ -629,9 +635,7 @@ class SummaryBarRenderer {
 
     public String renderRestartStatusHTML() {
         if (_helper == null) return "";
-        StringBuilder buf = new StringBuilder(512);
-        buf.append(_helper.getRestartStatus());
-        return buf.toString();
+        return _helper.getRestartStatus();
     }
 
     public String renderPeersHTML() {
@@ -784,9 +788,7 @@ class SummaryBarRenderer {
 
     public String renderFirewallAndReseedStatusHTML() {
         if (_helper == null) return "";
-        StringBuilder buf = new StringBuilder(512);
-        buf.append(_helper.getFirewallAndReseedStatus());
-        return buf.toString();
+        return _helper.getFirewallAndReseedStatus();
     }
 
     public String renderBandwidthHTML() {
@@ -962,9 +964,7 @@ class SummaryBarRenderer {
 
     public String renderDestinationsHTML() {
         if (_helper == null) return "";
-        StringBuilder buf = new StringBuilder(512);
-        buf.append(_helper.getDestinations());
-        return buf.toString();
+        return _helper.getDestinations();
     }
 
     /** @since 0.9.1 */
@@ -984,9 +984,6 @@ class SummaryBarRenderer {
                     entries = nmgr.getEntries();
             }
             if (!entries.isEmpty()) {
-                DateFormat fmt = DateFormat.getDateInstance(DateFormat.SHORT);
-                // the router sets the JVM time zone to UTC but saves the original here so we can get it
-                fmt.setTimeZone(SystemVersion.getSystemTimeZone(_context));
                 int i = 0;
                 // show a min of 1, max of 3, none older than 60 days over min
                 // Except, if news fetching is disabled, min is 0 and oldest is 7 days.
@@ -1015,9 +1012,8 @@ class SummaryBarRenderer {
                        .append(consoleNonce)
                        .append("\"");
                     if (entry.updated > 0) {
-                        Date date = new Date(entry.updated);
                         buf.append(" title=\"")
-                           .append(_t("Published")).append(": ").append(fmt.format(date)).append("\"");
+                           .append(_t("Published")).append(": ").append(DataHelper.formatDate(entry.updated)).append("\"");
                     }
                     buf.append(">");
                     buf.append(entry.title)
