@@ -220,8 +220,10 @@ public class SummaryHelper extends HelperBase {
     private NetworkStateMessage reachability() {
         if (_context.commSystem().isDummy())
             return new NetworkStateMessage(NetworkState.VMCOMM, "VM Comm System");
-        if (_context.router().getUptime() > 60*1000 && (!_context.router().gracefulShutdownInProgress()) &&
-            !_context.clientManager().isAlive())
+        if (_context.router().getUptime() > 60*1000 &&
+            !_context.clientManager().isAlive() &&
+            !_context.router().gracefulShutdownInProgress() &&
+            !_context.router().isRestarting())
             return new NetworkStateMessage(NetworkState.ERROR, _t("ERR-Client Manager I2CP Error - check logs"));  // not a router problem but the user should know
         // Warn based on actual skew from peers, not update status, so if we successfully offset
         // the clock, we don't complain.
@@ -245,10 +247,15 @@ public class SummaryHelper extends HelperBase {
             case IPV4_UNKNOWN_IPV6_OK:
             case IPV4_DISABLED_IPV6_OK:
             case IPV4_SNAT_IPV6_OK:
-                RouterAddress ra = routerInfo.getTargetAddress("NTCP");
-                if (ra == null)
+                List<RouterAddress> ras = routerInfo.getTargetAddresses("NTCP", "NTCP2");
+                if (ras.isEmpty())
                     return new NetworkStateMessage(NetworkState.RUNNING, _t(status.toStatusString()));
-                byte[] ip = ra.getIP();
+                byte[] ip = null;
+                for (RouterAddress ra : ras) {
+                    ip = ra.getIP();
+                    if (ip != null)
+                        break;
+                }
                 if (ip == null)
                     return new NetworkStateMessage(NetworkState.ERROR, _t("ERR-Unresolved TCP Address"));
                 // TODO set IPv6 arg based on configuration?
@@ -285,7 +292,7 @@ public class SummaryHelper extends HelperBase {
             case IPV4_UNKNOWN_IPV6_FIREWALLED:
             case IPV4_DISABLED_IPV6_UNKNOWN:
             default:
-                ra = routerInfo.getTargetAddress("SSU");
+                RouterAddress ra = routerInfo.getTargetAddress("SSU");
                 if (ra == null && _context.router().getUptime() > 5*60*1000) {
                     if (getActivePeers() <= 0)
                         return new NetworkStateMessage(NetworkState.ERROR, _t("ERR-No Active Peers, Check Network Connection and Firewall"));
@@ -642,7 +649,7 @@ public class SummaryHelper extends HelperBase {
             TunnelPoolSettings out = _context.tunnelManager().getOutboundSettings(d.calculateHash());
             name = (out != null ? out.getDestinationNickname() : null);
             if (name == null)
-                name = d.calculateHash().toBase64().substring(0,6);
+                name = d.toBase32();
             else
                 name = _t(name);
         } else {
