@@ -30,6 +30,9 @@ class ConnectionPacketHandler {
 
     public static final int MAX_SLOW_START_WINDOW = 24;
     
+    // see tickets 1939 and 2584
+    private static final int IMMEDIATE_ACK_DELAY = 150;
+
     public ConnectionPacketHandler(I2PAppContext context) {
         _context = context;
         _log = context.logManager().getLog(ConnectionPacketHandler.class);
@@ -50,6 +53,7 @@ class ConnectionPacketHandler {
         boolean ok = verifyPacket(packet, con);
         if (!ok) {
             boolean isTooFast = con.getSendStreamId() <= 0;
+            // Apparently an i2pd bug... see verifyPacket()
             if ( (!packet.isFlagSet(Packet.FLAG_RESET)) && (!isTooFast) && (_log.shouldLog(Log.WARN)) )
                 _log.warn("Packet does NOT verify: " + packet + " on " + con);
             packet.releasePayload();
@@ -197,7 +201,8 @@ class ConnectionPacketHandler {
                 // TODO the 250 below _may_ be a big limiter in how fast local "loopback" connections
                 // can go, however if it goes too fast then we start choking which causes
                 // frequent stalls anyway.
-                con.setNextSendTime(_context.clock().now() + 250);
+                // see tickets 1939 and 2584
+                con.setNextSendTime(_context.clock().now() + IMMEDIATE_ACK_DELAY);
             } else {
                 int delay = con.getOptions().getSendAckDelay();
                 if (packet.isFlagSet(Packet.FLAG_DELAY_REQUESTED)) // delayed ACK requested
@@ -543,7 +548,7 @@ class ConnectionPacketHandler {
                     }
                 }
             } else {
-                // getting a lot of these - why? mostly/all for acks...
+                // Apparently an i2pd bug...
                 if (con.getSendStreamId() != packet.getReceiveStreamId()) {
                     if (_log.shouldLog(Log.WARN))
                         _log.warn("Packet received with the wrong reply stream id: " 
