@@ -3,6 +3,7 @@ package net.i2p.client.streaming.impl;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InterruptedIOException;
+import java.net.SocketTimeoutException;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
@@ -73,7 +74,7 @@ class MessageInputStream extends InputStream {
         _readyDataBlocks = new ArrayList<ByteArray>(4);
         _highestReadyBlockId = -1;
         _highestBlockId = -1;
-        _readTimeout = -1;
+        _readTimeout = I2PSocketOptionsImpl.DEFAULT_READ_TIMEOUT;
         _notYetReadyBlocks = new HashMap<Long, ByteArray>(4);
         _dataLock = new Object();
         _maxMessageSize = maxMessageSize;
@@ -264,7 +265,7 @@ class MessageInputStream extends InputStream {
      */
     public void setReadTimeout(int timeout) {
         if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Changing read timeout from " + _readTimeout + " to " + timeout);
+            _log.debug("Changing read timeout from " + _readTimeout + " to " + timeout + ": " + hashCode());
         _readTimeout = timeout; 
     }
     
@@ -285,18 +286,18 @@ class MessageInputStream extends InputStream {
                 buf.append(available);
                 buf.append(" blocks: ").append(_readyDataBlocks.size());
                 
-                buf.append(" not ready blocks: ");
+                buf.append(" not ready blocks: [");
                 long notAvailable = 0;
-                for (Long id : _notYetReadyBlocks.keySet()) {
-                    ByteArray ba = _notYetReadyBlocks.get(id);
+                for (Map.Entry<Long, ByteArray> e : _notYetReadyBlocks.entrySet()) {
+                    Long id = e.getKey();
+                    ByteArray ba = e.getValue();
                     buf.append(id).append(" ");
-                    
-                    if (ba != null)
-                        notAvailable += ba.getValid();
+                    notAvailable += ba.getValid();
                 }
                 
-                buf.append("not ready bytes: ").append(notAvailable);
+                buf.append("] not ready bytes: ").append(notAvailable);
                 buf.append(" highest ready block: ").append(_highestReadyBlockId);
+                buf.append(" ID: ").append(hashCode());
                 
                 _log.debug(buf.toString(), new Exception("Input stream closed"));
             }
@@ -352,10 +353,6 @@ class MessageInputStream extends InputStream {
                     cur++;
                     _highestReadyBlockId++;
                 }
-                                        // FIXME Javadocs for setReadTimeout() say we will throw
-                                        // an InterruptedIOException.
-                                        // Java throws a SocketTimeoutException.
-                                        // We do neither.
             } else {
                 // _notYetReadyBlocks size is limited in canAccept()
                 if (_locallyClosed) {
@@ -375,9 +372,8 @@ class MessageInputStream extends InputStream {
     }
     
     /**
-     *  On a read timeout, this returns -1
-     *  (doesn't throw SocketTimeoutException like Socket)
-     *  (doesn't throw InterruptedIOException like our javadocs say)
+     *  On a read timeout, this throws a SocketTimeoutException
+     *  as of 0.9.36. Prior to that, returned -1.
      */
     public int read() throws IOException {
         int read = read(_oneByte, 0, 1);
@@ -387,9 +383,8 @@ class MessageInputStream extends InputStream {
     }
     
     /**
-     *  On a read timeout, this returns 0
-     *  (doesn't throw SocketTimeoutException like Socket)
-     *  (doesn't throw InterruptedIOException like our javadocs say)
+     *  On a read timeout, this throws a SocketTimeoutException
+     *  as of 0.9.36. Prior to that, returned 0.
      */
     @Override
     public int read(byte target[]) throws IOException {
@@ -397,9 +392,8 @@ class MessageInputStream extends InputStream {
     }
     
     /**
-     *  On a read timeout, this returns 0
-     *  (doesn't throw SocketTimeoutException like Socket)
-     *  (doesn't throw InterruptedIOException like our javadocs say)
+     *  On a read timeout, this throws a SocketTimeoutException
+     *  as of 0.9.36. Prior to that, returned 0.
      */
     @Override
     public int read(byte target[], int offset, int length) throws IOException {
@@ -409,10 +403,13 @@ class MessageInputStream extends InputStream {
             expiration = readTimeout + System.currentTimeMillis();
         else
             expiration = -1;
+        // for speed
+        final boolean shouldDebug = _log.shouldDebug();
         synchronized (_dataLock) {
             if (_locallyClosed) throw new IOException("Input stream closed");
             throwAnyError();
-            for (int i = 0; i < length; i++) {
+            int i = 0;
+            while (i < length) {
                 if ( (_readyDataBlocks.isEmpty()) && (i == 0) ) {
                     // ok, we havent found anything, so lets block until we get 
                     // at least one byte
@@ -424,13 +421,13 @@ class MessageInputStream extends InputStream {
                         if ( (_notYetReadyBlocks.isEmpty()) && (_closeReceived) ) {
                             if (_log.shouldLog(Log.INFO))
                                 _log.info("read(...," + offset + ", " + length + ")[" + i 
-                                           + "] got EOF after " + _readTotal + " " + toString());
+                                           + "] got EOF after " + _readTotal + ": " + hashCode());
                             return -1;
                         } else {
                             if (readTimeout < 0) {
-                                if (_log.shouldLog(Log.DEBUG))
+                                if (shouldDebug)
                                     _log.debug("read(...," + offset+", " + length+ ")[" + i 
-                                               + "] with no timeout: " + toString());
+                                               + "] wait w/o timeout: " + hashCode());
                                 try {
                                     _dataLock.wait();
                                 } catch (InterruptedException ie) {
@@ -438,14 +435,14 @@ class MessageInputStream extends InputStream {
                                     ioe2.initCause(ie);
                                     throw ioe2;
                                 }
-                                if (_log.shouldLog(Log.DEBUG))
+                                if (shouldDebug)
                                     _log.debug("read(...," + offset+", " + length+ ")[" + i 
-                                               + "] with no timeout complete: " + toString());
+                                               + "] wait w/o timeout complete: " + hashCode());
                                 throwAnyError();
                             } else if (readTimeout > 0) {
-                                if (_log.shouldLog(Log.DEBUG))
+                                if (shouldDebug)
                                     _log.debug("read(...," + offset+", " + length+ ")[" + i 
-                                               + "] with timeout: " + readTimeout + ": " + toString());
+                                               + "] wait: " + readTimeout + ": " + hashCode());
                                 try {
                                     _dataLock.wait(readTimeout);
                                 } catch (InterruptedException ie) {
@@ -453,29 +450,25 @@ class MessageInputStream extends InputStream {
                                     ioe2.initCause(ie);
                                     throw ioe2;
                                 }
-                                if (_log.shouldLog(Log.DEBUG))
+                                if (shouldDebug)
                                     _log.debug("read(...," + offset+", " + length+ ")[" + i 
-                                               + "] with timeout complete: " + readTimeout + ": " + toString());
+                                               + "] wait complete: " + readTimeout + ": " + hashCode());
                                 throwAnyError();
                             } else { // readTimeout == 0
                                 // noop, don't block
-                                if (_log.shouldLog(Log.DEBUG))
+                                if (shouldDebug)
                                     _log.debug("read(...," + offset+", " + length+ ")[" + i 
-                                               + "] with nonblocking setup: " + toString());
-                                return i;
+                                               + "] nonblocking return: " + hashCode());
+                                return 0;
                             }
                             if (_readyDataBlocks.isEmpty()) {
                                 if (readTimeout > 0) {
                                     long remaining = expiration - System.currentTimeMillis();
                                     if (remaining <= 0) {
-                                        // FIXME Javadocs for setReadTimeout() say we will throw
-                                        // an InterruptedIOException.
-                                        // Java throws a SocketTimeoutException.
-                                        // We do neither.
                                         if (_log.shouldLog(Log.INFO))
                                             _log.info("read(...," + offset+", " + length+ ")[" + i 
-                                                       + "] expired: " + toString());
-                                        return i;
+                                                       + "] timed out: " + hashCode());
+                                        throw new SocketTimeoutException();
                                     } else {
                                         readTimeout = (int) remaining;
                                     }
@@ -483,38 +476,36 @@ class MessageInputStream extends InputStream {
                             }
                         }
                     }
-                    // we looped a few times then got data, so this pass doesnt count
-                    i--;
                 } else if (_readyDataBlocks.isEmpty()) {
-                    if (_log.shouldLog(Log.DEBUG))
+                    if (shouldDebug)
                         _log.debug("read(...," + offset+", " + length+ ")[" + i 
-                                   + "] no more ready blocks, returning");
+                                   + "] no more ready blocks, returning: " + hashCode());
                     return i;
                 } else {
                     // either was already ready, or we wait()ed and it arrived
                     ByteArray cur = _readyDataBlocks.get(0);
-                    byte rv = cur.getData()[cur.getOffset()+_readyDataBlockIndex];
-                    _readyDataBlockIndex++;
+                    int toRead = Math.min(cur.getValid() - _readyDataBlockIndex, length - i);
+                    System.arraycopy(cur.getData(), cur.getOffset() + _readyDataBlockIndex, target, offset + i, toRead);
+                    _readyDataBlockIndex += toRead;
                     if (cur.getValid() <= _readyDataBlockIndex) {
                         _readyDataBlockIndex = 0;
                         _readyDataBlocks.remove(0);
                     }
-                    _readTotal++;
-                    target[offset + i] = rv; // rv < 0 ? rv + 256 : rv
-                    if ( (_readyDataBlockIndex <= 3) || (_readyDataBlockIndex >= cur.getValid() - 5) ) {
-                        if (_log.shouldLog(Log.DEBUG))
+                    _readTotal += toRead;
+                    if (shouldDebug) {
                             _log.debug("read(...," + offset+", " + length+ ")[" + i 
-                                       + "] after ready data: readyDataBlockIndex=" + _readyDataBlockIndex 
+                                       + "] copied " + toRead + " after ready data: readyDataBlockIndex=" + _readyDataBlockIndex 
                                        + " readyBlocks=" + _readyDataBlocks.size()
-                                       + " readTotal=" + _readTotal);
+                                       + " readTotal=" + _readTotal + ": " + hashCode());
                     }
+                    i += toRead;
                     //if (removed) 
                     //    _cache.release(cur);
                 }
-            } // for (int i = 0; i < length; i++) {
+            } // while (i < length) {
         }  // synchronized (_dataLock)
         
-        if (_log.shouldLog(Log.DEBUG))
+        if (shouldDebug)
             _log.debug("read(byte[]," + offset + ',' + length + ") read fully; total read: " +_readTotal);
 
         return length;
@@ -535,7 +526,7 @@ class MessageInputStream extends InputStream {
             }
         }
         if (_log.shouldLog(Log.DEBUG))
-            _log.debug("available(): " + numBytes);
+            _log.debug("available(): " + numBytes + ": " + hashCode());
         
         return numBytes;
     }
@@ -595,16 +586,17 @@ class MessageInputStream extends InputStream {
                 available -= _readyDataBlockIndex;
                 buf.append(available);
                 buf.append(" blocks: ").append(_readyDataBlocks.size());
-                buf.append(" not ready blocks: ");
+                buf.append(" not ready blocks: [");
                 long notAvailable = 0;
-                for (Long id : _notYetReadyBlocks.keySet()) {
-                    ByteArray ba = _notYetReadyBlocks.get(id);
+                for (Map.Entry<Long, ByteArray> e : _notYetReadyBlocks.entrySet()) {
+                    Long id = e.getKey();
+                    ByteArray ba = e.getValue();
                     buf.append(id).append(" ");
-                    if (ba != null)
-                        notAvailable += ba.getValid();
+                    notAvailable += ba.getValid();
                 }
-                buf.append("not ready bytes: ").append(notAvailable);
+                buf.append("] not ready bytes: ").append(notAvailable);
                 buf.append(" highest ready block: ").append(_highestReadyBlockId);
+                buf.append(" ID: ").append(hashCode());
                 _log.debug(buf.toString());
             }
             //while (_readyDataBlocks.size() > 0)
