@@ -19,22 +19,36 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *  
- *  $Revision: 1.1 $
+ *  $Revision: 1.2 $
  */
 package i2p.susi.webmail.encoding;
 
+import java.io.IOException;
+
 /**
+ * Converted from Exception to IOException in 0.9.33
  * @author susi
  */
-public class EncodingException extends Exception {
+public class EncodingException extends IOException {
 	/**
 	 * Comment for <code>serialVersionUID</code>
 	 */
 	private static final long serialVersionUID = 1L;
 
-	EncodingException( String msg )
+	/**
+	 * @since public since 0.9.33, was package private
+	 */
+	public EncodingException( String msg )
 	{
 		super( msg );
 	}
 
+	/**
+	 * @since 0.9.33
+	 */
+	public EncodingException(String msg, Exception cause)
+	{
+		super(msg, cause);
+	}
+
 }
