@@ -23,6 +23,7 @@ class IntelInfoImpl extends CPUIDCPUInfo implements IntelCPUInfo
     private static boolean isIvyCompatible;
     private static boolean isHaswellCompatible;
     private static boolean isBroadwellCompatible;
+    private static boolean isSkylakeCompatible;
     
     // If modelString != null, the cpu is considered correctly identified.
     private static final String smodel = identifyCPU();
@@ -39,6 +40,11 @@ class IntelInfoImpl extends CPUIDCPUInfo implements IntelCPUInfo
     public boolean IsIvyCompatible(){ return isIvyCompatible; }
     public boolean IsHaswellCompatible(){ return isHaswellCompatible; }
     public boolean IsBroadwellCompatible(){ return isBroadwellCompatible; }
+    /**
+     * Supports the AVX-512 instrutions.
+     * @since 0.9.41
+     */
+    public boolean IsSkylakeCompatible() { return isSkylakeCompatible; }
 
     public String getCPUModelString() throws UnknownCPUException
     {
@@ -61,6 +67,10 @@ class IntelInfoImpl extends CPUIDCPUInfo implements IntelCPUInfo
         if (family == 15 || family == 6) {
             // Intel uses extended model for family = 15 or family = 6,
             // which is not what wikipedia says
+            // we construct the model from EAX as follows:
+            // ext. model is 5th byte, base model is 2nd byte
+            // So e.g. for a published CPUID value of "306C1"
+            // the model here would be 0x3c, it's a Haswell.
             model += CPUID.getCPUExtendedModel() << 4;
         }
         if (family == 15) {
@@ -439,15 +449,23 @@ class IntelInfoImpl extends CPUIDCPUInfo implements IntelCPUInfo
                     // See Haswell notes above
                     case 0x4e:
                     case 0x55:
-                    case 0x5e: {
+                    case 0x5c: // Apollo Lake
+                    case 0x5e:
+                    case 0x66: // Cannon Lake
+                    case 0x67: // Cannon Lake
+                    case 0x6c: // Apollo Lake
+                               {
                         CPUIDCPUInfo c = new CPUIDCPUInfo();
                         if (c.hasAVX2() && c.hasBMI1()  && c.hasBMI2() &&
                             c.hasFMA3() && c.hasMOVBE() && c.hasABM()) {
                             isSandyCompatible = true;
                             isIvyCompatible = true;
                             isHaswellCompatible = true;
-                            if (c.hasADX())
+                            if (c.hasADX()) {
                                 isBroadwellCompatible = true;
+                                if (c.hasAVX512())
+                                    isSkylakeCompatible = true;
+                            }
                             modelString = "Skylake Core i3/i5/i7";
                         } else {
                             // This processor is "corei" compatible, as we define it,
@@ -463,23 +481,37 @@ class IntelInfoImpl extends CPUIDCPUInfo implements IntelCPUInfo
                         break;
                     }
 
+
+                // TODO case 0x5f: // Goldmont / Atom
+                // TODO case 0x7a: // Gemini Lake
+                // TODO case 0x7e: // Ice Lake
+
+
                 // following are for extended model == 8 or 9
                 // most flags are set above
                 // isCoreiCompatible = true is the default
 
                     // Kaby Lake
                     // ref: https://github.com/InstLatx64/InstLatx64/commit/9d2ea1a9eb727868dc514900da9e2f175710f9bf
+                    // ref: https://github.com/InstLatx64/InstLatx64/blob/master/ChangeLog.htm
+                    // ref: https://github.com/coreboot/coreboot/blob/master/src/soc/intel/common/block/include/intelblocks/mp_init.h
                     // See Haswell notes above
-                    case 0x8e:
-                    case 0x9e: {
+                    case 0x8e: // also Whiskey Lake, Coffee Lake
+                    case 0x9e: // also Whiskey Lake, Coffee Lake
+                    case 0xa5: // Comet Lake
+                    case 0xa6: // Comet Lake
+                               {
                         CPUIDCPUInfo c = new CPUIDCPUInfo();
                         if (c.hasAVX2() && c.hasBMI1()  && c.hasBMI2() &&
                             c.hasFMA3() && c.hasMOVBE() && c.hasABM()) {
                             isSandyCompatible = true;
                             isIvyCompatible = true;
                             isHaswellCompatible = true;
-                            if (c.hasADX())
+                            if (c.hasADX()) {
                                 isBroadwellCompatible = true;
+                                if (c.hasAVX512())
+                                    isSkylakeCompatible = true;
+                            }
                             modelString = "Kaby Lake Core i3/i5/i7";
                         } else {
                             // This processor is "corei" compatible, as we define it,
