@@ -15,7 +15,6 @@ import net.i2p.crypto.CryptixAESEngine;
 import net.i2p.crypto.DSAEngine;
 import net.i2p.crypto.ElGamalEngine;
 import net.i2p.crypto.HMAC256Generator;
-import net.i2p.crypto.HMACGenerator;
 import net.i2p.crypto.KeyGenerator;
 import net.i2p.crypto.SHA256Generator;
 import net.i2p.crypto.SessionKeyManager;
@@ -76,7 +75,6 @@ public class I2PAppContext {
     private ElGamalEngine _elGamalEngine;
     private AESEngine _AESEngine;
     private LogManager _logManager;
-    private HMACGenerator _hmac;
     private HMAC256Generator _hmac256;
     private SHA256Generator _sha;
     protected Clock _clock; // overridden in RouterContext
@@ -95,7 +93,6 @@ public class I2PAppContext {
     private volatile boolean _elGamalEngineInitialized;
     private volatile boolean _AESEngineInitialized;
     private volatile boolean _logManagerInitialized;
-    private volatile boolean _hmacInitialized;
     private volatile boolean _hmac256Initialized;
     private volatile boolean _shaInitialized;
     protected volatile boolean _clockInitialized; // used in RouterContext
@@ -119,7 +116,7 @@ public class I2PAppContext {
     // split up big lock on this to avoid deadlocks
     private final Object _lock1 = new Object(), _lock2 = new Object(), _lock3 = new Object(), _lock4 = new Object(),
                          _lock5 = new Object(), _lock7 = new Object(), _lock8 = new Object(),
-                         _lock9 = new Object(), _lock10 = new Object(), _lock11 = new Object(), _lock12 = new Object(),
+                         _lock10 = new Object(), _lock11 = new Object(), _lock12 = new Object(),
                          _lock13 = new Object(), _lock14 = new Object(), _lock16 = new Object(),
                          _lock17 = new Object(), _lock18 = new Object(), _lock19 = new Object(), _lock20 = new Object();
 
@@ -735,29 +732,6 @@ public class I2PAppContext {
         }
     }
 
-    /** 
-     * There is absolutely no good reason to make this context specific, 
-     * other than for consistency, and perhaps later we'll want to 
-     * include some stats.
-     *
-     * DEPRECATED - non-standard and used only by SSU.
-     * To be moved from context to SSU.
-     */
-    public HMACGenerator hmac() { 
-        if (!_hmacInitialized)
-            initializeHMAC();
-        return _hmac;
-    }
-
-    private void initializeHMAC() {
-        synchronized (_lock9) {
-            if (_hmac == null) {
-                _hmac= new HMACGenerator(this);
-            }
-            _hmacInitialized = true;
-        }
-    }
-
     /**
      * Un-deprecated in 0.9.38
      */
