@@ -38,8 +38,11 @@ import net.i2p.client.I2PClient;
 import net.i2p.client.I2PSession;
 import net.i2p.client.I2PSessionException;
 import net.i2p.client.I2PSessionListener;
+import net.i2p.client.LookupResult;
+import net.i2p.crypto.EncType;
 import net.i2p.crypto.SigType;
 import net.i2p.data.Base32;
+import net.i2p.data.BlindData;
 import net.i2p.data.DataFormatException;
 import net.i2p.data.DataHelper;
 import net.i2p.data.Destination;
@@ -49,6 +52,7 @@ import net.i2p.data.PrivateKey;
 import net.i2p.data.Signature;
 import net.i2p.data.SigningPrivateKey;
 import net.i2p.data.SigningPublicKey;
+import net.i2p.data.i2cp.BlindingInfoMessage;
 import net.i2p.data.i2cp.DestLookupMessage;
 import net.i2p.data.i2cp.DestReplyMessage;
 import net.i2p.data.i2cp.GetBandwidthLimitsMessage;
@@ -88,9 +92,9 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
     /** who we are */
     private final Destination _myDestination;
     /** private key for decryption */
-    private final PrivateKey _privateKey;
+    private PrivateKey _privateKey;
     /** private key for signing */
-    private   /* final */   SigningPrivateKey _signingPrivateKey;
+    private SigningPrivateKey _signingPrivateKey;
     /** configuration options */
     private final Properties _options;
     /** this session's Id */
@@ -197,6 +201,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
     private volatile boolean _routerSupportsFastReceive;
     private volatile boolean _routerSupportsHostLookup;
     private volatile boolean _routerSupportsLS2;
+    private volatile boolean _routerSupportsBlindingInfo;
 
     protected static final int CACHE_MAX_SIZE = SystemVersion.isAndroid() ? 32 : 128;
     /**
@@ -205,12 +210,6 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
      */
     private static final Map<Object, Destination> _lookupCache = new LHMCache<Object, Destination>(CACHE_MAX_SIZE);
     private static final String MIN_HOST_LOOKUP_VERSION = "0.9.11";
-    private static final boolean TEST_LOOKUP = false;
-
-    /** SSL interface (only) @since 0.8.3 */
-    protected static final String PROP_ENABLE_SSL = "i2cp.SSL";
-    protected static final String PROP_USER = "i2cp.username";
-    protected static final String PROP_PW = "i2cp.password";
 
     /**
      * Use Unix domain socket (or similar) to connect to a router
@@ -224,6 +223,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
 
     private static final String MIN_FAST_VERSION = "0.9.4";
     private static final String MIN_LS2_VERSION = "0.9.38";
+    private static final String MIN_BLINDINFO_VERSION = "0.9.43";
 
     /** @param routerVersion as rcvd in the SetDateMessage, may be null for very old routers */
     void dateUpdated(String routerVersion) {
@@ -232,7 +232,6 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
                                      (routerVersion != null && routerVersion.length() > 0 &&
                                       VersionComparator.comp(routerVersion, MIN_FAST_VERSION) >= 0);
         _routerSupportsHostLookup = isrc ||
-                                    TEST_LOOKUP ||
                                      (routerVersion != null && routerVersion.length() > 0 &&
                                       VersionComparator.comp(routerVersion, MIN_HOST_LOOKUP_VERSION) >= 0);
         _routerSupportsSubsessions = isrc ||
@@ -241,6 +240,9 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
         _routerSupportsLS2 = isrc ||
                                      (routerVersion != null && routerVersion.length() > 0 &&
                                       VersionComparator.comp(routerVersion, MIN_LS2_VERSION) >= 0);
+        _routerSupportsBlindingInfo = isrc ||
+                                     (routerVersion != null && routerVersion.length() > 0 &&
+                                      VersionComparator.comp(routerVersion, MIN_BLINDINFO_VERSION) >= 0);
         synchronized (_stateLock) {
             if (_state == State.OPENING) {
                 changeState(State.GOTDATE);
@@ -251,6 +253,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
     public static final int LISTEN_PORT = I2PClient.DEFAULT_LISTEN_PORT;
 
     private static final int BUF_SIZE = 32*1024;
+    private static final SessionId DUMMY_SESSION = new SessionId(65535);
 
     /**
      * for extension by SimpleSession (no dest)
@@ -423,12 +426,12 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
         // auto-add auth if required, not set in the options, and we are not in the same JVM
         if ((!_context.isRouterContext()) &&
             _context.getBooleanProperty("i2cp.auth") &&
-            ((!opts.containsKey(PROP_USER)) || (!opts.containsKey(PROP_PW)))) {
-            String configUser = _context.getProperty(PROP_USER);
-            String configPW = _context.getProperty(PROP_PW);
+            ((!opts.containsKey(I2PClient.PROP_USER)) || (!opts.containsKey(I2PClient.PROP_PW)))) {
+            String configUser = _context.getProperty(I2PClient.PROP_USER);
+            String configPW = _context.getProperty(I2PClient.PROP_PW);
             if (configUser != null && configPW != null) {
-                options.setProperty(PROP_USER, configUser);
-                options.setProperty(PROP_PW, configPW);
+                options.setProperty(I2PClient.PROP_USER, configUser);
+                options.setProperty(I2PClient.PROP_PW, configPW);
             }
         }
         if (options.getProperty(I2PClient.PROP_FAST_RECEIVE) == null)
@@ -587,6 +590,12 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
      */
     private void readDestination(InputStream destKeyStream) throws DataFormatException, IOException {
         _myDestination.readBytes(destKeyStream);
+        // we don't support EncTypes in Destinations,
+        // but i2pd does, and also, this code is used from PrivateKeyFile to
+        // read in router.keys.dat files and we will support EncTypes for RouterIdentities
+        EncType etype = _myDestination.getPublicKey().getType();
+        if (etype != EncType.ELGAMAL_2048)
+            _privateKey = new PrivateKey(etype);
         _privateKey.readBytes(destKeyStream);
         SigType dtype = _myDestination.getSigningPublicKey().getType();
         _signingPrivateKey = new SigningPrivateKey(dtype);
@@ -734,7 +743,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
                         } catch (InvocationTargetException e) {
                             throw new I2PSessionException("Cannot load DomainSocketFactory", e);
                         }
-                    } else if (Boolean.parseBoolean(_options.getProperty(PROP_ENABLE_SSL))) {
+                    } else if (Boolean.parseBoolean(_options.getProperty(I2PClient.PROP_ENABLE_SSL))) {
                         try {
                             I2PSSLSocketFactory fact = new I2PSSLSocketFactory(_context, false, "certificates/i2cp");
                             _socket = fact.createSocket(_hostname, _portNum);
@@ -762,12 +771,12 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
             _reader.startReading();
             if (_log.shouldLog(Log.DEBUG)) _log.debug(getPrefix() + "Before getDate");
             Properties auth = null;
-            if ((!_context.isRouterContext()) && _options.containsKey(PROP_USER) && _options.containsKey(PROP_PW)) {
+            if ((!_context.isRouterContext()) && _options.containsKey(I2PClient.PROP_USER) && _options.containsKey(I2PClient.PROP_PW)) {
                 // Only supported by routers 0.9.11 or higher, but we don't know the version yet.	
                 // Auth will also be sent in the SessionConfig.
                 auth = new OrderedProperties();
-                auth.setProperty(PROP_USER, _options.getProperty(PROP_USER));
-                auth.setProperty(PROP_PW, _options.getProperty(PROP_PW));
+                auth.setProperty(I2PClient.PROP_USER, _options.getProperty(I2PClient.PROP_USER));
+                auth.setProperty(I2PClient.PROP_PW, _options.getProperty(I2PClient.PROP_PW));
             }
             sendMessage_unchecked(new GetDateMessage(CoreVersion.VERSION, auth));
             waitForDate();
@@ -1493,6 +1502,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
             if (h.equals(w.hash)) {
                 synchronized (w) {
                     w.destination = d;
+                    w.code = LookupResult.RESULT_SUCCESS;
                     w.notifyAll();
                 }
             }
@@ -1508,6 +1518,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
         for (LookupWaiter w : _pendingLookups) {
             if (h.equals(w.hash)) {
                 synchronized (w) {
+                    w.code = LookupResult.RESULT_FAILURE;
                     w.notifyAll();
                 }
             }
@@ -1532,6 +1543,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
                 }
                 synchronized (w) {
                     w.destination = d;
+                    w.code = LookupResult.RESULT_SUCCESS;
                     w.notifyAll();
                 }
             }
@@ -1543,10 +1555,11 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
      *  on reception of HostReplyMessage
      *  @since 0.9.11
      */
-    void destLookupFailed(long nonce) {
+    void destLookupFailed(long nonce, int code) {
         for (LookupWaiter w : _pendingLookups) {
             if (nonce == w.nonce) {
                 synchronized (w) {
+                    w.code = code;
                     w.notifyAll();
                 }
             }
@@ -1574,6 +1587,11 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
         public final long nonce;
         /** the reply; synch on this */
         public Destination destination;
+        /**
+         *  the return code; sync on this
+         *  @since 0.9.43
+         */
+        public int code;
 
         public LookupWaiter(Hash h) {
             this(h, -1);
@@ -1592,6 +1610,16 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
             this.name = name;
             this.nonce = nonce;
         }
+
+        /** Dummy, completed
+         *  @since 0.9.43
+         */
+        public LookupWaiter(Destination d) {
+            hash = null;
+            name = null;
+            nonce = 0;
+            destination = d;
+        }
     }
 
     /** @since 0.9.20 */
@@ -1650,7 +1678,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
                     _log.info("Sending HostLookup for " + h);
                 SessionId id = _sessionId;
                 if (id == null)
-                    id = new SessionId(65535);
+                    id = DUMMY_SESSION;
                 sendMessage_unchecked(new HostLookupMessage(id, h, nonce, maxWait));
             } else {
                 if (_log.shouldLog(Log.INFO))
@@ -1701,12 +1729,50 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
      *  @return null on failure
      */
     public Destination lookupDest(String name, long maxWait) throws I2PSessionException {
+        LookupWaiter waiter = x_lookupDest(name, maxWait);
+        if (waiter == null)
+            return null;
+        synchronized(waiter) {
+            return waiter.destination;
+        }
+    }
+
+    /**
+     *  Ask the router to lookup a Destination by host name.
+     *  Blocking. See above for details.
+     *  Same as lookupDest() but with a failure code in the return value
+     *
+     *  @param maxWait ms
+     *  @since 0.9.43
+     *  @return non-null
+     */
+    public LookupResult lookupDest2(String name, long maxWait) throws I2PSessionException {
+        LookupWaiter waiter = x_lookupDest(name, maxWait);
+        if (waiter == null)
+            return new LkupResult(LookupResult.RESULT_FAILURE, null);
+        synchronized(waiter) {
+            int code = waiter.code;
+            Destination d = waiter.destination;
+            if (d == null && code == LookupResult.RESULT_SUCCESS)
+                code = LookupResult.RESULT_FAILURE;
+            return new LkupResult(code, d);
+        }
+    }
+
+    /**
+     *  Ask the router to lookup a Destination by host name.
+     *  Blocking. See above for details.
+     *  @param maxWait ms
+     *  @since 0.9.11
+     *  @return null on failure
+     */
+    private LookupWaiter x_lookupDest(String name, long maxWait) throws I2PSessionException {
         if (name.length() == 0)
             return null;
         // Shortcut for b64
         if (name.length() >= 516) {
             try {
-                return new Destination(name);
+                return new LookupWaiter(new Destination(name));
             } catch (DataFormatException dfe) {
                 return null;
             }
@@ -1717,7 +1783,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
         synchronized (_lookupCache) {
             Destination rv = _lookupCache.get(name);
             if (rv != null)
-                return rv;
+                return new LookupWaiter(rv);
         }
         if (isClosed()) {
             if (_log.shouldLog(Log.INFO))
@@ -1727,7 +1793,7 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
         if (!_routerSupportsHostLookup) {
             // do them a favor and convert to Hash lookup
             if (name.length() == 60 && name.toLowerCase(Locale.US).endsWith(".b32.i2p"))
-                return lookupDest(Hash.create(Base32.decode(name.toLowerCase(Locale.US).substring(0, 52))), maxWait);
+                return new LookupWaiter(lookupDest(Hash.create(Base32.decode(name.toLowerCase(Locale.US).substring(0, 52))), maxWait));
             // else unsupported
             if (_log.shouldLog(Log.WARN))
                 _log.warn("Router does not support HostLookup for " + name);
@@ -1736,18 +1802,17 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
         int nonce = _lookupID.incrementAndGet() & 0x7fffffff;
         LookupWaiter waiter = new LookupWaiter(name, nonce);
         _pendingLookups.offer(waiter);
-        Destination rv = null;
         try {
             if (_log.shouldLog(Log.INFO))
                 _log.info("Sending HostLookup for " + name);
             SessionId id = _sessionId;
             if (id == null)
-                id = new SessionId(65535);
+                id = DUMMY_SESSION;
             sendMessage_unchecked(new HostLookupMessage(id, name, nonce, maxWait));
             try {
                 synchronized (waiter) {
                     waiter.wait(maxWait);
-                    rv = waiter.destination;
+                    return waiter;
                 }
             } catch (InterruptedException ie) {
                 throw new I2PSessionException("Interrupted", ie);
@@ -1755,7 +1820,6 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
         } finally {
             _pendingLookups.remove(waiter);
         }
-        return rv;
     }
 
     /**
@@ -1786,6 +1850,21 @@ public abstract class I2PSessionImpl implements I2PSession, I2CPMessageReader.I2
         return _bwLimits;
     }
 
+    /**
+     *
+     *  @since 0.9.43
+     */
+    public void sendBlindingInfo(BlindData bd) throws I2PSessionException {
+        if (!_routerSupportsBlindingInfo)
+            throw new I2PSessionException("Router does not support BlindingInfo");
+        if (_log.shouldInfo())
+            _log.info("Sending BlindingInfo");
+        SessionId id = _sessionId;
+        if (id == null)
+            id = DUMMY_SESSION;
+        sendMessage_unchecked(new BlindingInfoMessage(bd, id));
+    }
+
     protected void updateActivity() {
         _lastActivity = _context.clock().now();
         if (_isReduced) {
