@@ -34,9 +34,6 @@ import net.i2p.data.Hash;
 class LookupDest {
 
     private static final long DEFAULT_TIMEOUT = 15*1000;
-    private static final String PROP_ENABLE_SSL = "i2cp.SSL";
-    private static final String PROP_USER = "i2cp.username";
-    private static final String PROP_PW = "i2cp.password";
 
     protected LookupDest(I2PAppContext context) {}
 
@@ -110,15 +107,15 @@ class LookupDest {
             s = ctx.getProperty(I2PClient.PROP_TCP_PORT);
             if (s != null)
                 opts.put(I2PClient.PROP_TCP_PORT, s);
-            s = ctx.getProperty(PROP_ENABLE_SSL);
+            s = ctx.getProperty(I2PClient.PROP_ENABLE_SSL);
             if (s != null)
-                opts.put(PROP_ENABLE_SSL, s);
-            s = ctx.getProperty(PROP_USER);
+                opts.put(I2PClient.PROP_ENABLE_SSL, s);
+            s = ctx.getProperty(I2PClient.PROP_USER);
             if (s != null)
-                opts.put(PROP_USER, s);
-            s = ctx.getProperty(PROP_PW);
+                opts.put(I2PClient.PROP_USER, s);
+            s = ctx.getProperty(I2PClient.PROP_PW);
             if (s != null)
-                opts.put(PROP_PW, s);
+                opts.put(I2PClient.PROP_PW, s);
         }
         return opts;
     }
