@@ -15,15 +15,21 @@ package net.i2p.crypto;
  * put("MessageDigest.SHA-1", "com.bitzi.util.SHA1");
  */
 //package com.bitzi.util;
-import java.security.*;
-//--+---+1--+---+--2+---+---+3--+---+--4+---+---+5--+---+--6+---+---+7--+---+--
-//34567890123456789012345678901234567890123456789012345678901234567890123456789
- 
+
+import java.security.DigestException;
+import java.security.MessageDigest;
+import java.security.NoSuchAlgorithmException;
+
+import net.i2p.util.SystemVersion;
+
 /**
+ * NOTE: As of 0.8.7, use getInstance() instead of new SHA1(), which will
+ * return the JVM's MessageDigest if it is faster.
+ *
  * <p>The FIPS PUB 180-2 standard specifies four secure hash algorithms (SHA-1,
  * SHA-256, SHA-384 and SHA-512) for computing a condensed representation of
- * electronic data (message).  When a message of any length < 2^^64 bits (for
- * SHA-1 and SHA-256) or < 2^^128 bits (for SHA-384 and SHA-512) is input to
+ * electronic data (message).  When a message of any length &lt; 2^^64 bits (for
+ * SHA-1 and SHA-256) or &lt; 2^^128 bits (for SHA-384 and SHA-512) is input to
  * an algorithm, the result is an output called a message digest.  The message
  * digests range in length from 160 to 512 bits, depending on the algorithm.
  * Secure hash algorithms are typically used with other cryptographic
@@ -56,7 +62,7 @@ import java.security.*;
  *      http://www.itl.nist.gov/div897/pubs/fip180-1.htm</a></li>
  *   <li> Bruce Schneier, "Section 18.7 Secure Hash Algorithm (SHA)",
  *      <cite>Applied Cryptography, 2nd edition</cite>, <br>
- *      John Wiley & Sons, 1996</li>
+ *      John Wiley &amp; Sons, 1996</li>
  * </ol>
  */
 public final class SHA1 extends MessageDigest implements Cloneable {
@@ -64,7 +70,7 @@ public final class SHA1 extends MessageDigest implements Cloneable {
     /**
      * This implementation returns a fixed-size digest.
      */
-    private static final int HASH_LENGTH = 20; // bytes == 160 bits
+    static final int HASH_LENGTH = 20; // bytes == 160 bits
  
     /**
      * Private context for incomplete blocks and padding bytes.
@@ -86,8 +92,25 @@ public final class SHA1 extends MessageDigest implements Cloneable {
      */
     private int hA, hB, hC, hD, hE;
  
+    private static final boolean _useBitzi;
+    static {
+        // oddly, Bitzi is faster than Oracle - see test results below
+        boolean useBitzi = true;
+        if (SystemVersion.isApache() ||            // Harmony
+            SystemVersion.isGNU()) {               // JamVM or gij
+            try {
+                MessageDigest.getInstance("SHA-1");
+                useBitzi = false;
+            } catch (NoSuchAlgorithmException e) {}
+        }
+        //if (useBitzi)
+        //    System.out.println("INFO: Using Bitzi SHA-1");
+        _useBitzi = useBitzi;
+    }
+
     /**
      * Creates a SHA1 object with default initial state.
+     * NOTE: Use getInstance() to get the fastest implementation.
      */
     public SHA1() {
         super("SHA-1");
@@ -95,12 +118,26 @@ public final class SHA1 extends MessageDigest implements Cloneable {
         init();
     }
  
+    /**
+     *  @return the fastest digest, either new SHA1() or MessageDigest.getInstance("SHA-1")
+     *  @since 0.8.7
+     */
+    public static MessageDigest getInstance() {
+        if (!_useBitzi) {
+            try {
+                return MessageDigest.getInstance("SHA-1");
+            } catch (NoSuchAlgorithmException e) {}
+        }
+        return new SHA1();
+    }
+
     /**
      * Clones this object.
      */
+    @Override
     public Object clone() throws CloneNotSupportedException {
         SHA1 that = (SHA1)super.clone();
-        that.pad = (byte[])this.pad.clone();
+        that.pad = this.pad.clone();
         return that;
     }
  
@@ -114,6 +151,7 @@ public final class SHA1 extends MessageDigest implements Cloneable {
      * <code>java.security.MessageDigestSpi</code>.
      * @return the digest length in bytes.
      */
+    @Override
     public int engineGetDigestLength() {
         return HASH_LENGTH;
     }
@@ -124,6 +162,7 @@ public final class SHA1 extends MessageDigest implements Cloneable {
      * Overrides the protected abstract method of
      * <code>java.security.MessageDigestSpi</code>.
      */
+    @Override
     protected void engineReset() {
         int i = 60;
         do {
@@ -180,6 +219,7 @@ public final class SHA1 extends MessageDigest implements Cloneable {
      * @param offset  the offset to start from in the array of bytes.
      * @param len  the number of bytes to use, starting at offset.
      */
+    @Override
     public void engineUpdate(byte[] input, int offset, int len) {
         if (offset >= 0 && len >= 0 && offset + len <= input.length) {
             bytes += len;
@@ -235,6 +275,7 @@ public final class SHA1 extends MessageDigest implements Cloneable {
      * java.security.MessageDigestSpi.
      * @return the length of the digest stored in the output buffer.
      */
+    @Override
     public byte[] engineDigest() {
         try {
             final byte hashvalue[] = new byte[HASH_LENGTH];
@@ -265,6 +306,7 @@ public final class SHA1 extends MessageDigest implements Cloneable {
      *             length.
      * @return  the length of the digest stored in the output buffer.
      */
+    @Override
     public int engineDigest(byte[] hashvalue, int offset, final int len)
             throws DigestException {
         if (len >= HASH_LENGTH) {
@@ -694,4 +736,65 @@ public final class SHA1 extends MessageDigest implements Cloneable {
         hD += d;
         hC += /* c= */ (c << 30) | (c >>> 2);
     }
+
+    //private static final int RUNS = 100000;
+
+    /**
+     *  Test the GNU and the JVM's implementations for speed
+     *
+     *  Results: 2011-05 eeepc Atom
+     *  <pre>
+     *  JVM	strlen	GNU ms	JVM  ms 
+     *	Oracle	387	  1406	 2357
+     *	Oracle	 40	   522	  475
+     *	Harmony	387	  5504	 3474
+     *	Harmony	 40	  4396	 1593
+     *	JamVM	387	 25578	21966
+     *	JamVM	 40	  5380	 4195
+     *	gij	387	 47225	 3501
+     *	gij	 40	  9861    919
+     *  </pre>
+     *
+     *  @since 0.8.7
+     */
+/****
+    public static void main(String args[]) {
+        if (args.length <= 0) {
+            System.err.println("Usage: SHA1 string");
+            return;
+        }
+
+        byte[] data = args[0].getBytes();
+        SHA1 gnu = new SHA1();
+        long start = System.currentTimeMillis();
+        for (int i = 0; i < RUNS; i++) {
+            gnu.update(data, 0, data.length);
+            byte[] sha = gnu.digest();
+            if (i == 0)
+                System.out.println("SHA1 [" + args[0] + "] = [" + Base64.encode(sha) + "]");
+            gnu.reset();
+        }
+        long time = System.currentTimeMillis() - start;
+        System.out.println("Time for " + RUNS + " SHA-256 computations:");
+        System.out.println("GNU time (ms): " + time);
+
+        start = System.currentTimeMillis();
+        MessageDigest md;
+        try {
+            md = MessageDigest.getInstance("SHA-1");
+        } catch (NoSuchAlgorithmException e) {
+            System.err.println("Fatal: " + e);
+            return;
+        }
+        for (int i = 0; i < RUNS; i++) {
+            md.reset();
+            byte[] sha = md.digest(data);
+            if (i == 0)
+                System.out.println("SHA1 [" + args[0] + "] = [" + Base64.encode(sha) + "]");
+        }
+        time = System.currentTimeMillis() - start;
+
+        System.out.println("JVM time (ms): " + time);
+    }
+****/
 }
