@@ -14,6 +14,7 @@ import java.security.PublicKey;
 import java.security.cert.CertificateFactory;
 import java.security.cert.X509Certificate;
 import java.security.cert.X509CRL;
+import java.security.spec.InvalidKeySpecException;
 import java.security.spec.X509EncodedKeySpec;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
@@ -32,6 +33,7 @@ import javax.crypto.spec.DHPublicKeySpec;
 import javax.security.auth.x500.X500Principal;
 
 import static net.i2p.crypto.SigUtil.intToASN1;
+import net.i2p.crypto.eddsa.EdDSAPublicKey;
 import net.i2p.data.DataHelper;
 import net.i2p.data.Signature;
 import net.i2p.data.SigningPrivateKey;
@@ -86,6 +88,10 @@ public final class SelfSignedGenerator {
     private static final String OID_POLICY_ANY = "2.5.29.32.0";
     // Authority Key Identifier
     private static final String OID_AKI = "2.5.29.35";
+    // Extended Key Usage
+    private static final String OID_EKU = "2.5.29.37";
+    // ID-KP-ServerAuth
+    private static final String OID_ID_KP_SERVERAUTH = "1.3.6.1.5.5.7.3.1";
 
     private static final Map<String, String> OIDS;
     static {
@@ -245,8 +251,20 @@ public final class SelfSignedGenerator {
         // some simple tests
         PublicKey cpub = cert.getPublicKey();
         cert.verify(cpub);
-        if (!cpub.equals(jpub))
-            throw new GeneralSecurityException("pubkey mismatch");
+        if (!cpub.equals(jpub)) {
+            boolean ok = false;
+            if ((jpub instanceof EdDSAPublicKey) &&
+                cpub.getClass().getName().equals("sun.security.x509.X509Key")) {
+                // X509Certificate will sometimes contain an X509Key rather than the EdDSAPublicKey itself; the contained
+                // key is valid but needs to be instanced as an EdDSAPublicKey before it can be used.
+                try {
+                    cpub = new EdDSAPublicKey(new X509EncodedKeySpec(cpub.getEncoded()));
+                    ok = cpub.equals(jpub);
+                } catch (InvalidKeySpecException ex) {}
+            }
+            if (!ok)
+                throw new GeneralSecurityException("pubkey mismatch, in: " + jpub.getClass() + " cert: " + cpub.getClass());
+        }
         // todo crl tests
 
         Object[] rv = { jpub, jpriv, cert, crl };
@@ -362,10 +380,10 @@ public final class SelfSignedGenerator {
         byte[] version = { (byte) 0xa0, 3, 2, 1, 2 };
 
         // positive serial number (long)
-        byte[] serial = new byte[10];
+        byte[] serial = new byte[11];
         serial[0] = 2;
-        serial[1] = 8;
-        RandomSource.getInstance().nextBytes(serial, 2, 8);
+        serial[1] = 9;
+        RandomSource.getInstance().nextBytes(serial, 2, 9);
         serial[2] &= 0x7f;
 
         // going to use this for both issuer and subject
@@ -389,6 +407,7 @@ public final class SelfSignedGenerator {
         // ProviderException thrown here
         byte[] pubbytes = jpub.getEncoded();
         byte[] extbytes = getExtensions(pubbytes, cname, altNames);
+        //System.out.println("Extensions:\n" + HexDump.dump(extbytes));
 
         int len = version.length + serial.length + sigoid.length + issuer.length +
                   validity.length + subject.length + pubbytes.length + extbytes.length;
@@ -609,6 +628,8 @@ public final class SelfSignedGenerator {
         byte[] oid7 = getEncodedOID(OID_POLICY_ANY);
         byte[] oid8 = getEncodedOID(OID_QT_UNOTICE);
         byte[] oid9 = getEncodedOID(OID_QT_CPSURI);
+        byte[] oid10 = getEncodedOID(OID_EKU);
+        byte[] oid11 = getEncodedOID(OID_ID_KP_SERVERAUTH);
         byte[] TRUE = new byte[] { 1, 1, (byte) 0xff };
 
         // extXlen does NOT include the 0x30 and length
@@ -667,9 +688,12 @@ public final class SelfSignedGenerator {
         int wrap68len = spaceFor(wrap67len); // Policies seq
         int ext6len = oid6.length + spaceFor(wrap68len); // OID + octet string
 
+        int wrap7len = spaceFor(oid11.length); // EKU OID
+        int ext7len = oid10.length + spaceFor(wrap7len); // EKU
+
         int extslen = spaceFor(ext1len) + spaceFor(ext2len) + spaceFor(ext4len) + spaceFor(ext5len);
         if (isCA)
-            extslen += spaceFor(ext3len) + spaceFor(ext6len);
+            extslen += spaceFor(ext3len) + spaceFor(ext6len) + spaceFor(ext7len);
         int seqlen = spaceFor(extslen);
         int totlen = spaceFor(seqlen);
         byte[] rv = new byte[totlen];
@@ -808,6 +832,20 @@ public final class SelfSignedGenerator {
             idx += policyTextBytes.length;
         }
 
+        // EKU
+        if (isCA) {
+            rv[idx++] = (byte) 0x30;
+            idx = intToASN1(rv, idx, ext7len);
+            System.arraycopy(oid10, 0, rv, idx, oid10.length);
+            idx += oid10.length;
+            rv[idx++] = (byte) 0x04;  // octet string wraps a sequence
+            idx = intToASN1(rv, idx, wrap7len);
+            rv[idx++] = (byte) 0x30;  // seq.
+            idx = intToASN1(rv, idx, oid11.length);
+            System.arraycopy(oid11, 0, rv, idx, oid11.length);
+            idx += oid11.length;
+        }
+
         return rv;
     }
 
