@@ -26,6 +26,7 @@ import java.io.PrintWriter;
 import java.io.UnsupportedEncodingException;
 import java.math.BigInteger;
 import java.security.MessageDigest;
+import java.text.DateFormat;
 import java.text.DecimalFormat;
 import java.util.Arrays;
 import java.util.Collection;
@@ -38,6 +39,7 @@ import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Properties;
+import java.util.TimeZone;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.regex.Pattern;
 import java.util.zip.Deflater;
@@ -107,6 +109,14 @@ public class DataHelper {
     private static final Pattern ILLEGAL_KEY =  Pattern.compile("[#=\r\n;]");
     private static final Pattern ILLEGAL_VALUE =  Pattern.compile("[#\r\n]");
 
+    /**
+     *  The default formatting for date/time, current locale, local time zone
+     *  @since 0.9.43
+     */
+    private static final DateFormat DATE_FORMAT = DateFormat.getDateInstance(DateFormat.MEDIUM);
+    private static final DateFormat TIME_FORMAT = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.SHORT);
+    private static boolean _date_tz_set, _time_tz_set;
+
     /** Read a mapping from the stream, as defined by the I2P data structure spec,
      * and store it into a Properties object.
      *
@@ -514,7 +524,7 @@ public class DataHelper {
             fos = new SecureFileOutputStream(tmpFile);
             out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(fos, "UTF-8")));
             out.println("# NOTE: This I2P config file must use UTF-8 encoding");
-            out.println("# Last saved: " + new Date(System.currentTimeMillis()));
+            out.println("# Last saved: " + formatTime(System.currentTimeMillis()));
             for (Map.Entry<Object, Object> entry : props.entrySet()) {
                 String name = (String) entry.getKey();
                 String val = (String) entry.getValue();
@@ -1608,6 +1618,46 @@ public class DataHelper {
             default: return bytes + space;
         }
     }
+
+    /**
+     *  The default formatting for date, current locale, local time zone.
+     *  Warning - NOT UTC!
+     *  Examples:
+     *  en: Aug 30, 2019
+     *  de: 30.08.2019
+     *  @since 0.9.43
+     */
+    public static String formatDate(long now) {
+        synchronized(DATE_FORMAT) {
+            if (!_date_tz_set) {
+                // delayed set, too early if done in static block
+                TimeZone tz = SystemVersion.getSystemTimeZone();
+                DATE_FORMAT.setTimeZone(tz);
+                _date_tz_set = true;
+            }
+            return DATE_FORMAT.format(new Date(now));
+        }
+    }
+
+    /**
+     *  The default formatting for date/time, current locale, local time zone.
+     *  Warning - NOT UTC!
+     *  Examples:
+     *  en: Aug 30, 2019 12:38 PM
+     *  de: 30.08.2019 12:38
+     *  @since 0.9.43
+     */
+    public static String formatTime(long now) {
+        synchronized(TIME_FORMAT) {
+            if (!_time_tz_set) {
+                // delayed set, too early if done in static block
+                TimeZone tz = SystemVersion.getSystemTimeZone();
+                TIME_FORMAT.setTimeZone(tz);
+                _time_tz_set = true;
+            }
+            return TIME_FORMAT.format(new Date(now));
+        }
+    }
     
     /**
      * Strip out any HTML (simply removing any less than / greater than symbols)
