@@ -7,6 +7,7 @@ import java.io.InputStream;
 import java.io.OutputStream;
 import java.util.Collections;
 import java.util.List;
+import java.util.Set;
 
 import net.i2p.I2PAppContext;
 import net.i2p.crypto.Blinding;
@@ -26,8 +27,6 @@ import net.i2p.util.Log;
  *
  * PRELIMINARY - Subject to change - see proposal 123
  *
- * Per-client auth TODO
- *
  * @since 0.9.38
  */
 public class EncryptedLeaseSet extends LeaseSet2 {
@@ -134,6 +133,20 @@ public class EncryptedLeaseSet extends LeaseSet2 {
         return super.getEncryptionKeys();
     }
 
+    /**
+     *  If more than one key, return the first supported one.
+     *  If none supported, return null.
+     *
+     *  @return first supported key or null
+     *  @since 0.9.44
+     */
+    @Override
+    public PublicKey getEncryptionKey(Set<EncType> supported) {
+        if (_decryptedLS2 != null)
+            return _decryptedLS2.getEncryptionKey(supported);
+        return super.getEncryptionKey(supported);
+    }
+
     /**
      * Overridden to set the blinded key.
      * setSecret() MUST be called before this for non-null secret, or alpha will be wrong.
@@ -401,6 +414,7 @@ public class EncryptedLeaseSet extends LeaseSet2 {
         // inner LS is always unpublished
         int saveFlags = _flags;
         setUnpublished();
+        setBlindedWhenPublished();
         try {
             // Inner layer - type - data covered by sig
             baos.write(KEY_TYPE_LS2);
@@ -498,7 +512,7 @@ public class EncryptedLeaseSet extends LeaseSet2 {
                     off += ID_LEN;
                     ChaCha20.encrypt(clientKey, clientIVandID, authcookie, 0, ciphertext, off, authcookie.length);
                     if (_log.shouldDebug()) {
-                        _log.debug("Added client ID/enc.cookie:\n" +
+                        _log.debug("DH: Added client ID/enc.cookie:\n" +
                                    net.i2p.util.HexDump.dump(clientIVandID, IV_LEN, ID_LEN) +
                                    net.i2p.util.HexDump.dump(ciphertext, off, COOKIE_LEN) +
                                    "for client key:\n" +
@@ -518,7 +532,7 @@ public class EncryptedLeaseSet extends LeaseSet2 {
                 byte[] clientIVandID = new byte[32];
                 for (SimpleDataStructure sds : clientKeys) {
                     if (!(sds instanceof PrivateKey))
-                        throw new IllegalArgumentException("Bad DH client key type: " + sds);
+                        throw new IllegalArgumentException("Bad PSK client key type: " + sds);
                     PrivateKey csk = (PrivateKey) sds;
                     if (csk.getType() != EncType.ECIES_X25519)
                         throw new IllegalArgumentException("Bad PSK client key type: " + csk);
@@ -531,7 +545,7 @@ public class EncryptedLeaseSet extends LeaseSet2 {
                     off += ID_LEN;
                     ChaCha20.encrypt(clientKey, clientIVandID, authcookie, 0, ciphertext, off, authcookie.length);
                     if (_log.shouldDebug()) {
-                        _log.debug("Added client key:\n" +
+                        _log.debug("PSK: Added client key:\n" +
                                    net.i2p.util.HexDump.dump(clientKey) +
                                    "client IV:\n:" +
                                    net.i2p.util.HexDump.dump(clientIVandID, 0, IV_LEN) +
@@ -642,7 +656,7 @@ public class EncryptedLeaseSet extends LeaseSet2 {
                 throw new DataFormatException("No client entries");
             authLen = 1 + SALT_LEN + 2 + (count * CLIENT_LEN);
             if (_log.shouldDebug()) {
-                 _log.debug("Found " + count + " client entries, authsalt is:\n" +
+                 _log.debug("Auth type " + authType + ", found " + count + " client entries, authsalt is:\n" +
                             net.i2p.util.HexDump.dump(seed));
             }
             byte[] clientKey = new byte[32];
