@@ -10,15 +10,13 @@ package net.i2p.data.i2cp;
  */
 
 import net.i2p.I2PException;
-import net.i2p.util.Log;
 
 /**
- * Represent an error serializing or deserializing an APIMessage
+ * Represent an error serializing or deserializing a message
  *
  * @author jrandom
  */
 public class I2CPMessageException extends I2PException {
-    private final static Log _log = new Log(I2CPMessageException.class);
 
     public I2CPMessageException(String message, Throwable parent) {
         super(message, parent);
@@ -27,4 +25,4 @@ public class I2CPMessageException extends I2PException {
     public I2CPMessageException(String message) {
         super(message);
     }
-}
\ No newline at end of file
+}
