@@ -11,7 +11,9 @@ import net.i2p.data.ByteArray;
  * needs to be done (e.g. write(foo); toByteArray();), call releaseBuffer
  * to put the buffer back into the cache.
  * 
+ * @deprecated unused
  */
+@Deprecated
 public class CachingByteArrayOutputStream extends ByteArrayOutputStream {
     private ByteCache _cache;
     private ByteArray _buf;
