@@ -15,11 +15,16 @@ import net.i2p.CoreVersion;
  *
  */
 public class RouterVersion {
-    public final static String ID = "$Revision: 1.467 $ $Date: 2006-10-08 20:45:02 $";
-    public final static String VERSION = "0.6.1.26";
+    /** deprecated */
+    public final static String ID = "Monotone";
+    public final static String VERSION = CoreVersion.VERSION;
     public final static long BUILD = 0;
+
+    /** for example "-test" */
+    public final static String EXTRA = "";
+    public final static String FULL_VERSION = VERSION + "-" + BUILD + EXTRA;
     public static void main(String args[]) {
-        System.out.println("I2P Router version: " + VERSION + "-" + BUILD);
+        System.out.println("I2P Router version: " + FULL_VERSION);
         System.out.println("Router ID: " + RouterVersion.ID);
         System.out.println("I2P Core version: " + CoreVersion.VERSION);
         System.out.println("Core ID: " + CoreVersion.ID);
