@@ -93,8 +93,23 @@ class LookupDestJob extends JobImpl {
                             SigningPublicKey spk = bd.getUnblindedPubKey();
                             BlindData bd2 = getContext().netDb().getBlindData(spk);
                             if (bd2 != null) {
-                                bd = bd2;
+                                // BlindData from database may have privkey or secret
+                                // check if we need it but don't have it
+                                if ((bd.getAuthRequired() && bd2.getAuthPrivKey() == null) ||
+                                    (bd.getSecretRequired() && (bd2.getSecret() == null || bd2.getSecret().length() == 0))) {
+                                    // don't copy over existing info, this will force an immediate
+                                    // failure in runJob()
+                                    if (_log.shouldDebug())
+                                        _log.debug("No auth or secret, immediate fail " + bd);
+                                } else {
+                                    bd = bd2;
+                                }
                             } else {
+                                long now = getContext().clock().now();
+                                bd.setDate(now);
+                                long exp = now + ((bd.getAuthRequired() || bd.getSecretRequired()) ? 365*24*60*60*1000L
+                                                                                                   :  90*24*68*60*1000L);
+                                bd.setExpiration(exp);
                                 getContext().netDb().setBlindData(bd);
                             }
                             h = bd.getBlindedHash();
@@ -125,10 +140,27 @@ class LookupDestJob extends JobImpl {
 
     public void runJob() {
         if (_blindData != null) {
+            boolean fail1 = _blindData.getAuthRequired() && _blindData.getAuthPrivKey() == null;
+            boolean fail2 = _blindData.getSecretRequired() &&
+                            (_blindData.getSecret() == null || _blindData.getSecret().length() == 0);
+            if (fail1 || fail2) {
+                int code;
+                if (fail1 && fail2)
+                    code = HostReplyMessage.RESULT_SECRET_AND_KEY_REQUIRED;
+                else if (fail1)
+                    code = HostReplyMessage.RESULT_KEY_REQUIRED;
+                else
+                    code = HostReplyMessage.RESULT_SECRET_REQUIRED;
+                if (_log.shouldDebug())
+                    _log.debug("Failed b33 lookup " + _blindData.getUnblindedPubKey() + " with code " + code);
+                returnFail(code);
+            }
+            // do this after the fail checks above, because even if we
+            // have the dest, it won't help get a LS.
             Destination d = _blindData.getDestination();
             if (d != null) {
                 if (_log.shouldDebug())
-                    _log.debug("Found cached b33 lookup " + _name + " to " + d);
+                    _log.debug("Found cached b33 lookup " + _blindData.getUnblindedPubKey() + " to " + d);
                 returnDest(d);
                 return;
             }
@@ -147,11 +179,15 @@ class LookupDestJob extends JobImpl {
             }
         } else if (_hash != null) {
             DoneJob done = new DoneJob(getContext());
+            // shorten timeout so we can respond before the client side times out
+            long timeout = _timeout;
+            if (timeout > 1500)
+                timeout -= 500;
             // TODO tell router this is an encrypted lookup, skip 38 or earlier ffs?
-            getContext().netDb().lookupDestination(_hash, done, _timeout, _fromLocalDest);
+            getContext().netDb().lookupDestination(_hash, done, timeout, _fromLocalDest);
         } else {
             // blinding decode fail
-            returnFail();
+            returnFail(HostReplyMessage.RESULT_DECRYPTION_FAILURE);
         }
     }
 
@@ -198,13 +234,22 @@ class LookupDestJob extends JobImpl {
     }
 
     /**
-     *  Return the failed hash so the client can correlate replies with requests
+     *  Return the request ID or failed hash so the client can correlate replies with requests
      *  @since 0.8.3
      */
     private void returnFail() {
+        returnFail(HostReplyMessage.RESULT_FAILURE);
+    }
+
+    /**
+     *  Return the request ID or failed hash so the client can correlate replies with requests
+     *  @param code failure code, greater than zero, only used for HostReplyMessage
+     *  @since 0.9.43
+     */
+    private void returnFail(int code) {
         I2CPMessage msg;
         if (_reqID >= 0)
-            msg = new HostReplyMessage(_sessID, HostReplyMessage.RESULT_FAILURE, _reqID);
+            msg = new HostReplyMessage(_sessID, code, _reqID);
         else if (_hash != null)
             msg = new DestReplyMessage(_hash);
         else
