@@ -14,17 +14,23 @@ import java.util.Date;
 import java.util.HashSet;
 import java.util.Set;
 
+import net.i2p.crypto.EncType;
 import net.i2p.crypto.SessionKeyManager;
+import net.i2p.data.Certificate;
 import net.i2p.data.DataFormatException;
 import net.i2p.data.DataHelper;
 import net.i2p.data.Hash;
+import net.i2p.data.PrivateKey;
 import net.i2p.data.PublicKey;
 import net.i2p.data.SessionKey;
 import net.i2p.data.SessionTag;
 import net.i2p.data.i2np.GarlicClove;
 import net.i2p.data.i2np.GarlicMessage;
 import net.i2p.data.i2np.I2NPMessage;
+import net.i2p.router.LeaseSetKeys;
 import net.i2p.router.RouterContext;
+import net.i2p.router.crypto.ratchet.MuxedSKM;
+import net.i2p.router.crypto.ratchet.RatchetSKM;
 import net.i2p.util.Log;
 
 /**
@@ -34,10 +40,14 @@ import net.i2p.util.Log;
 public class GarlicMessageBuilder {
 
     /**
+     *  ELGAMAL_2048 only.
+     *
      *  @param local non-null; do not use this method for the router's SessionKeyManager
      *  @param minTagOverride 0 for no override, &gt; 0 to override SKM's settings
      */
     static boolean needsTags(RouterContext ctx, PublicKey key, Hash local, int minTagOverride) {
+        if (key.getType() == EncType.ECIES_X25519)
+            return false;
         SessionKeyManager skm = ctx.clientManager().getClientSessionKeyManager(local);
         if (skm == null)
             return true;
@@ -51,6 +61,7 @@ public class GarlicMessageBuilder {
     
     /**
      * Unused and probably a bad idea.
+     * ELGAMAL_2048 only.
      *
      * Used below only on a recursive call if the garlic message contains a garlic message.
      * We don't need the SessionKey or SesssionTags returned
@@ -60,6 +71,7 @@ public class GarlicMessageBuilder {
      *
      * @param ctx scope
      * @param config how/what to wrap
+     * @return null if expired
      * @throws IllegalArgumentException on error
      */
     private static GarlicMessage buildMessage(RouterContext ctx, GarlicConfig config) {
@@ -70,14 +82,17 @@ public class GarlicMessageBuilder {
 
     /**
      * Now unused, since we have to generate a reply token first in OCMOSJ but we don't know if tags are required yet.
+     * ELGAMAL_2048 only.
      *
      * @param ctx scope
      * @param config how/what to wrap
-     * @param wrappedKey output parameter that will be filled with the sessionKey used
+     * @param wrappedKey non-null with null data,
+     *                   output parameter that will be filled with the SessionKey used
      * @param wrappedTags Output parameter that will be filled with the sessionTags used.
                           If non-empty on return you must call skm.tagsDelivered() when sent
                           and then call skm.tagsAcked() or skm.failTags() later.
      * @param skm non-null
+     * @return null if expired
      * @throws IllegalArgumentException on error
      */
     public static GarlicMessage buildMessage(RouterContext ctx, GarlicConfig config, SessionKey wrappedKey, Set<SessionTag> wrappedTags,
@@ -94,11 +109,13 @@ public class GarlicMessageBuilder {
     ***/
 
     /**
-     * called by OCMJH
+     * ELGAMAL_2048 only.
+     * Called by OCMJH.
      *
      * @param ctx scope
      * @param config how/what to wrap
-     * @param wrappedKey output parameter that will be filled with the sessionKey used
+     * @param wrappedKey non-null with null data,
+     *                   output parameter that will be filled with the SessionKey used
      * @param wrappedTags Output parameter that will be filled with the sessionTags used.
                           If non-empty on return you must call skm.tagsDelivered() when sent
                           and then call skm.tagsAcked() or skm.failTags() later.
@@ -106,6 +123,7 @@ public class GarlicMessageBuilder {
                                Set to zero to disable tag delivery. You must set to zero if you are not
                                equipped to confirm delivery and call skm.tagsAcked() or skm.failTags() later.
      * @param skm non-null
+     * @return null if expired
      * @throws IllegalArgumentException on error
      */
     public static GarlicMessage buildMessage(RouterContext ctx, GarlicConfig config, SessionKey wrappedKey, Set<SessionTag> wrappedTags,
@@ -114,11 +132,13 @@ public class GarlicMessageBuilder {
     }
 
     /**
-     * called by netdb and above
+     * ELGAMAL_2048 only.
+     * Called by netdb and above.
      *
      * @param ctx scope
      * @param config how/what to wrap
-     * @param wrappedKey output parameter that will be filled with the sessionKey used
+     * @param wrappedKey non-null with null data,
+     *                   output parameter that will be filled with the SessionKey used
      * @param wrappedTags Output parameter that will be filled with the sessionTags used.
                           If non-empty on return you must call skm.tagsDelivered() when sent
                           and then call skm.tagsAcked() or skm.failTags() later.
@@ -128,6 +148,7 @@ public class GarlicMessageBuilder {
                                If this is always 0, it forces ElGamal every time.
      * @param lowTagsThreshold the threshold
      * @param skm non-null
+     * @return null if expired
      * @throws IllegalArgumentException on error
      */
     public static GarlicMessage buildMessage(RouterContext ctx, GarlicConfig config, SessionKey wrappedKey, Set<SessionTag> wrappedTags,
@@ -144,14 +165,14 @@ public class GarlicMessageBuilder {
             } else
                 key = config.getRecipient().getIdentity().getPublicKey();
         }
+        if (key.getType() != EncType.ELGAMAL_2048)
+            throw new IllegalArgumentException();
         
         if (log.shouldLog(Log.INFO))
             log.info("Encrypted with public key to expire on " + new Date(config.getExpiration()));
         
         SessionKey curKey = skm.getCurrentOrNewKey(key);
-        SessionTag curTag = null;
-
-            curTag = skm.consumeNextAvailableTag(key, curKey);
+        SessionTag curTag = skm.consumeNextAvailableTag(key, curKey);
             
             if (log.shouldLog(Log.DEBUG)) {
                 int availTags = skm.getAvailableTags(key, curKey);
@@ -167,25 +188,26 @@ public class GarlicMessageBuilder {
 
         wrappedKey.setData(curKey.getData());
         
-        return buildMessage(ctx, config, wrappedKey, wrappedTags, key, curKey, curTag);
+        return buildMessage(ctx, config, wrappedTags, key, curKey, curTag);
     }
     
     /**
-     *  used by TestJob and directly above
-     *  and for encrypting DatabaseLookupMessages
+     *  ELGAMAL_2048 only.
+     *  Used by TestJob, and directly above,
+     *  and by MessageWrapper for encrypting DatabaseLookupMessages
      *
      * @param ctx scope
      * @param config how/what to wrap
-     * @param wrappedKey unused - why??
      * @param wrappedTags New tags to be sent along with the message.
      *                    200 max enforced at receiver; null OK
      * @param target public key of the location being garlic routed to (may be null if we 
      *               know the encryptKey and encryptTag)
      * @param encryptKey sessionKey used to encrypt the current message, non-null
      * @param encryptTag sessionTag used to encrypt the current message, null to force ElG
+     * @return null if expired
      * @throws IllegalArgumentException on error
      */
-    public static GarlicMessage buildMessage(RouterContext ctx, GarlicConfig config, SessionKey wrappedKey, Set<SessionTag> wrappedTags,
+    public static GarlicMessage buildMessage(RouterContext ctx, GarlicConfig config, Set<SessionTag> wrappedTags,
                                              PublicKey target, SessionKey encryptKey, SessionTag encryptTag) {
         Log log = ctx.logManager().getLog(GarlicMessageBuilder.class);
         if (config == null)
@@ -199,6 +221,11 @@ public class GarlicMessageBuilder {
         
         // TODO - 128 is the minimum padded size - should it be more? less? random?
         byte encData[] = ctx.elGamalAESEngine().encrypt(cloveSet, target, encryptKey, wrappedTags, encryptTag, 128);
+        if (encData == null) {
+            if (log.shouldWarn())
+                log.warn("ElG encrypt fail");
+            return null;
+        }
         msg.setData(encData);
         msg.setMessageExpiration(config.getExpiration());
         
@@ -216,6 +243,69 @@ public class GarlicMessageBuilder {
         return msg;
     }
     
+    /**
+     * ECIES_X25519 only.
+     * Called by GarlicMessageBuilder only.
+     *
+     * @param ctx scope
+     * @param config how/what to wrap
+     * @param target public key of the location being garlic routed to (may be null if we 
+     *               know the encryptKey and encryptTag)
+     * @return null if expired or on other errors
+     * @throws IllegalArgumentException on error
+     * @since 0.9.44
+     */
+    static GarlicMessage buildECIESMessage(RouterContext ctx, GarlicConfig config,
+                                           PublicKey target, Hash from, SessionKeyManager skm) {
+        PublicKey key = config.getRecipientPublicKey();
+        if (key.getType() != EncType.ECIES_X25519)
+            throw new IllegalArgumentException();
+        Log log = ctx.logManager().getLog(GarlicMessageBuilder.class);
+        GarlicMessage msg = new GarlicMessage(ctx);
+        CloveSet cloveSet = buildECIESCloveSet(ctx, config);
+        LeaseSetKeys lsk = ctx.keyManager().getKeys(from);
+        if (lsk == null) {
+            if (log.shouldWarn())
+                log.warn("No LSK for " + from.toBase32());
+            return null;
+        }
+        PrivateKey priv = lsk.getDecryptionKey(EncType.ECIES_X25519);
+        if (priv == null) {
+            if (log.shouldWarn())
+                log.warn("No key for " + from.toBase32());
+            return null;
+        }
+
+        RatchetSKM rskm;
+        if (skm instanceof RatchetSKM) {
+            rskm = (RatchetSKM) skm;
+        } else if (skm instanceof MuxedSKM) {
+            rskm = ((MuxedSKM) skm).getECSKM();
+        } else {
+            if (log.shouldWarn())
+                log.warn("No SKM for " + from.toBase32());
+            return null;
+        }
+        byte encData[] = ctx.eciesEngine().encrypt(cloveSet, target, priv, rskm);
+        if (encData == null) {
+            if (log.shouldWarn())
+                log.warn("Encrypt fail for " + from.toBase32());
+            return null;
+        }
+        msg.setData(encData);
+        msg.setMessageExpiration(config.getExpiration());
+        long timeFromNow = config.getExpiration() - ctx.clock().now();
+        if (timeFromNow < 1*1000) {
+            if (log.shouldDebug())
+                log.debug("Building a message expiring in " + timeFromNow + "ms: " + config, new Exception("created by"));
+            return null;
+        }
+        if (log.shouldDebug())
+            log.debug("CloveSet (" + config.getCloveCount() + " cloves) for message " + msg.getUniqueId()
+                     + " encrypted message data is " + encData.length + " bytes");
+        return msg;
+    }
+    
 /****
     private static void noteWrap(RouterContext ctx, GarlicMessage wrapper, GarlicConfig contained) {
         for (int i = 0; i < contained.getCloveCount(); i++) {
@@ -237,8 +327,7 @@ public class GarlicMessageBuilder {
      * @throws IllegalArgumentException on error
      */
     private static byte[] buildCloveSet(RouterContext ctx, GarlicConfig config) {
-        ByteArrayOutputStream baos = null;
-        Log log = ctx.logManager().getLog(GarlicMessageBuilder.class);
+        ByteArrayOutputStream baos;
         try {
             if (config instanceof PayloadGarlicConfig) {
                 byte clove[] = buildClove(ctx, (PayloadGarlicConfig)config);
@@ -253,7 +342,7 @@ public class GarlicMessageBuilder {
                         //log.debug("Subclove IS a payload garlic clove");
                         cloves[i] = buildClove(ctx, (PayloadGarlicConfig)c);
                     } else {
-                        log.debug("Subclove IS NOT a payload garlic clove");
+                        //log.debug("Subclove IS NOT a payload garlic clove");
                         // See notes below
                         cloves[i] = buildClove(ctx, c);
                     }
@@ -271,16 +360,14 @@ public class GarlicMessageBuilder {
             DataHelper.writeLong(baos, 4, config.getId());
             DataHelper.writeLong(baos, DataHelper.DATE_LENGTH, config.getExpiration());
         } catch (IOException ioe) {
-            log.error("Error building the clove set", ioe);
             throw new IllegalArgumentException("Error building the clove set", ioe);
         } catch (DataFormatException dfe) {
-            log.error("Error building the clove set", dfe);
             throw new IllegalArgumentException("Error building the clove set", dfe);
         }
         return baos.toByteArray();
     }
     
-    private static byte[] buildClove(RouterContext ctx, PayloadGarlicConfig config) throws DataFormatException, IOException {
+    private static byte[] buildClove(RouterContext ctx, PayloadGarlicConfig config) {
         GarlicClove clove = new GarlicClove(ctx);
         clove.setData(config.getPayload());
         return buildCommonClove(clove, config);
@@ -311,17 +398,53 @@ public class GarlicMessageBuilder {
         return buildCommonClove(clove, config);
     }
     
-    private static byte[] buildCommonClove(GarlicClove clove, GarlicConfig config) throws DataFormatException, IOException {
+    private static byte[] buildCommonClove(GarlicClove clove, GarlicConfig config) {
         clove.setCertificate(config.getCertificate());
         clove.setCloveId(config.getId());
         clove.setExpiration(new Date(config.getExpiration()));
         clove.setInstructions(config.getDeliveryInstructions());
         return clove.toByteArray();
-        /*
-        int size = clove.estimateSize();
-        ByteArrayOutputStream baos = new ByteArrayOutputStream(size);
-        clove.writeBytes(baos);
-        return baos.toByteArray();
-         */
+    }
+    
+    /**
+     * Build the unencrypted CloveSet specified by the config.
+     * Unlike for Elgamal, the cloves do not contain a unique
+     * ID and expiration, and the CloveSet does not contain
+     * a unique certificate, ID, or expiration date.
+     *
+     * @throws IllegalArgumentException on error
+     * @since 0.9.44
+     */
+    private static CloveSet buildECIESCloveSet(RouterContext ctx, GarlicConfig config) {
+        GarlicClove[] arr;
+        if (config instanceof PayloadGarlicConfig) {
+            GarlicClove clove = buildECIESClove(ctx, (PayloadGarlicConfig)config);
+            arr = new GarlicClove[1];
+            arr[0] = clove;
+        } else {
+            int cnt = config.getCloveCount();
+            arr = new GarlicClove[cnt];
+            for (int i = 0; i < cnt; i++) {
+                GarlicConfig c = config.getClove(i);
+                if (c instanceof PayloadGarlicConfig) {
+                    arr[i] = buildECIESClove(ctx, (PayloadGarlicConfig)c);
+                } else {
+                    throw new IllegalArgumentException("Subclove IS NOT a payload garlic clove");
+                }
+            }
+        }
+        // GarlicConfig cert, ID, and expiration all ignored here
+        CloveSet rv = new CloveSet(arr, Certificate.NULL_CERT, config.getId(), config.getExpiration());
+        return rv;
+    }
+    
+    private static GarlicClove buildECIESClove(RouterContext ctx, PayloadGarlicConfig config) {
+        GarlicClove clove = new GarlicClove(ctx);
+        clove.setData(config.getPayload());
+        clove.setCertificate(Certificate.NULL_CERT);
+        clove.setCloveId(0);
+        clove.setExpiration(new Date(config.getExpiration()));
+        clove.setInstructions(config.getDeliveryInstructions());
+        return clove;
     }
 }
