@@ -944,11 +944,16 @@ public abstract class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacad
         
         // spoof / hash collision detection
         // todo allow non-exp to overwrite exp
-        if (rv != null && !leaseSet.getDestination().equals(rv.getDestination()))
-            throw new IllegalArgumentException("LS Hash collision");
+        if (rv != null) {
+            Destination d1 = leaseSet.getDestination();
+            Destination d2 = rv.getDestination();
+            if (d1 != null && d2 != null && !d1.equals(d2))
+                throw new IllegalArgumentException("LS Hash collision");
+        }
 
         EncryptedLeaseSet encls = null;
-        if (leaseSet.getType() == DatabaseEntry.KEY_TYPE_ENCRYPTED_LS2) {
+        int type = leaseSet.getType();
+        if (type == DatabaseEntry.KEY_TYPE_ENCRYPTED_LS2) {
             // set dest or key before validate() calls verifySignature() which
             // will do the decryption
             encls = (EncryptedLeaseSet) leaseSet;
@@ -994,6 +999,14 @@ public abstract class KademliaNetworkDatabaseFacade extends NetworkDatabaseFacad
                 store(dest.getHash(), decls);
                 _blindCache.setBlinded(dest);
             }
+        } else if (type == DatabaseEntry.KEY_TYPE_LS2 || type == DatabaseEntry.KEY_TYPE_META_LS2) {
+             // if it came in via garlic
+             LeaseSet2 ls2 = (LeaseSet2) leaseSet;
+             if (ls2.isBlindedWhenPublished()) {
+                 Destination dest = leaseSet.getDestination();
+                 if (dest != null)
+                    _blindCache.setBlinded(dest, null, null);
+            }
         }
 
         // Iterate through the old failure / success count, copying over the old
