@@ -9,53 +9,55 @@ package net.i2p.router.networkdb.kademlia;
  */
 
 import java.util.ArrayList;
-import java.util.Iterator;
+import java.util.Collections;
 import java.util.List;
 import java.util.Set;
 
+import net.i2p.data.DatabaseEntry;
 import net.i2p.data.DataHelper;
-import net.i2p.data.DataStructure;
 import net.i2p.data.Hash;
 import net.i2p.data.LeaseSet;
-import net.i2p.data.RouterInfo;
 import net.i2p.data.TunnelId;
 import net.i2p.data.i2np.DatabaseLookupMessage;
 import net.i2p.data.i2np.DatabaseSearchReplyMessage;
 import net.i2p.data.i2np.DatabaseStoreMessage;
+import net.i2p.data.i2np.I2NPMessage;
+import net.i2p.data.router.RouterInfo;
 import net.i2p.router.Job;
 import net.i2p.router.JobImpl;
+import net.i2p.router.OutNetMessage;
 import net.i2p.router.RouterContext;
 import net.i2p.router.TunnelInfo;
 import net.i2p.router.message.SendMessageDirectJob;
-import net.i2p.stat.Rate;
-import net.i2p.stat.RateStat;
 import net.i2p.util.Log;
 
 /**
  * Search for a particular key iteratively until we either find a value or we 
  * run out of peers
  *
+ * Note that this is rarely if ever used directly, and is primary used by the ExploreJob extension.
+ * FloodOnlySearchJob and FloodSearchJob do not extend this.
+ * It also does not update peer profile stats.
  */
 class SearchJob extends JobImpl {
-    private Log _log;
-    protected KademliaNetworkDatabaseFacade _facade;
-    private SearchState _state;
-    private Job _onSuccess;
-    private Job _onFailure;
-    private long _expiration;
-    private long _timeoutMs;
-    private boolean _keepStats;
-    private boolean _isLease;
+    protected final Log _log;
+    protected final KademliaNetworkDatabaseFacade _facade;
+    private final SearchState _state;
+    private final Job _onSuccess;
+    private final Job _onFailure;
+    private final long _expiration;
+    private final long _timeoutMs;
+    private final boolean _keepStats;
+    private final boolean _isLease;
     private Job _pendingRequeueJob;
-    private PeerSelector _peerSelector;
-    private List _deferredSearches;
+    private final PeerSelector _peerSelector;
+    private final List<Search> _deferredSearches;
     private boolean _deferredCleared;
     private long _startedOn;
     private boolean _floodfillPeersExhausted;
     private int _floodfillSearchesOutstanding;
     
     private static final int SEARCH_BREDTH = 3; // 10 peers at a time 
-    private static final int SEARCH_PRIORITY = 400; // large because the search is probably for a real search
     /** only send the 10 closest "dont tell me about" refs */
     static final int MAX_CLOSEST = 10;
     
@@ -79,16 +81,21 @@ class SearchJob extends JobImpl {
      *
      */
     private static final long REQUEUE_DELAY = 1000;
+
+    // TODO pass to the tunnel dispatcher
+    //private final static int LOOKUP_PRIORITY = OutNetMessage.PRIORITY_MY_NETDB_LOOKUP;
+    //private final static int STORE_PRIORITY = OutNetMessage.PRIORITY_HIS_NETDB_STORE;
     
     /**
      * Create a new search for the routingKey specified
      * 
      */
-    public SearchJob(RouterContext context, KademliaNetworkDatabaseFacade facade, Hash key, Job onSuccess, Job onFailure, long timeoutMs, boolean keepStats, boolean isLease) {
+    public SearchJob(RouterContext context, KademliaNetworkDatabaseFacade facade, Hash key,
+                     Job onSuccess, Job onFailure, long timeoutMs, boolean keepStats, boolean isLease) {
         super(context);
         if ( (key == null) || (key.getData() == null) ) 
-            throw new IllegalArgumentException("Search for null key?  wtf");
-        _log = getContext().logManager().getLog(SearchJob.class);
+            throw new IllegalArgumentException("Search for null key?");
+        _log = getContext().logManager().getLog(getClass());
         _facade = facade;
         _state = new SearchState(getContext(), key);
         _onSuccess = onSuccess;
@@ -96,16 +103,13 @@ class SearchJob extends JobImpl {
         _timeoutMs = timeoutMs;
         _keepStats = keepStats;
         _isLease = isLease;
-        _deferredSearches = new ArrayList(0);
-        _deferredCleared = false;
+        _deferredSearches = new ArrayList<Search>(0);
         _peerSelector = facade.getPeerSelector();
         _startedOn = -1;
-        _floodfillPeersExhausted = false;
-        _floodfillSearchesOutstanding = 0;
         _expiration = getContext().clock().now() + timeoutMs;
-        getContext().statManager().addRateData("netDb.searchCount", 1, 0);
+        getContext().statManager().addRateData("netDb.searchCount", 1);
         if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Search (" + getClass().getName() + " for " + key.toBase64(), new Exception("Search enqueued by"));
+            _log.debug("Search (" + getClass().getName() + " for " + key, new Exception("Search enqueued by"));
     }
 
     public void runJob() {
@@ -123,12 +127,23 @@ class SearchJob extends JobImpl {
     
     private static final boolean DEFAULT_FLOODFILL_ONLY = true;
     
+    /** this is now misnamed, as it is only used to determine whether to return floodfill peers only */
     static boolean onlyQueryFloodfillPeers(RouterContext ctx) {
-        if (isCongested(ctx))
-            return true;
-        return Boolean.valueOf(ctx.getProperty("netDb.floodfillOnly", DEFAULT_FLOODFILL_ONLY + "")).booleanValue();
+        //if (isCongested(ctx))
+        //    return true;
+        // If we are floodfill, we want the FloodfillPeerSelector (in add()) to include
+        // non-ff peers (if required) in DatabaseSearchReplyMessage responses
+        // so that Exploration works.
+        // ExploreJob is disabled if we are floodfill.
+        // The other two places this was called (one below and one in FNDF)
+        // have been commented out.
+        // Returning false essentially enables kademlia as a backup to floodfill for search responses.
+        if (ctx.netDb().floodfillEnabled())
+            return false;
+        return ctx.getProperty("netDb.floodfillOnly", DEFAULT_FLOODFILL_ONLY);
     }
     
+/***
     static boolean isCongested(RouterContext ctx) {
         float availableSend = ctx.bandwidthLimiter().getOutboundKBytesPerSecond()*1024 - ctx.bandwidthLimiter().getSendBps();
         float availableRecv = ctx.bandwidthLimiter().getInboundKBytesPerSecond()*1024 - ctx.bandwidthLimiter().getReceiveBps();
@@ -136,8 +151,11 @@ class SearchJob extends JobImpl {
         // in that range without a problem
         return ( (availableSend < 6*1024) || (availableRecv < 6*1024) );
     }
+***/
     
+    /** timeout */
     static final int PER_FLOODFILL_PEER_TIMEOUT = 10*1000;
+    static final long MIN_TIMEOUT = 2500;
     
     protected int getPerPeerTimeoutMs(Hash peer) {
         int timeout = 0;
@@ -148,7 +166,7 @@ class SearchJob extends JobImpl {
         long now = getContext().clock().now();
         
         if (now + timeout > _expiration)
-            return (int)(_expiration - now);
+            return (int) Math.max(_expiration - now, MIN_TIMEOUT);
         else
             return timeout;
     }
@@ -189,22 +207,29 @@ class SearchJob extends JobImpl {
                 _log.debug(getJobId() + ": Already completed");
             return;
         }
+        if (_state.isAborted()) {
+            if (_log.shouldLog(Log.INFO))
+                _log.info(getJobId() + ": Search aborted");
+            _state.complete();
+            fail();
+            return;
+        }
         if (_log.shouldLog(Log.INFO))
             _log.info(getJobId() + ": Searching: " + _state);
         if (isLocal()) {
             if (_log.shouldLog(Log.INFO))
                 _log.info(getJobId() + ": Key found locally");
-            _state.complete(true);
+            _state.complete();
             succeed();
         } else if (isExpired()) {
             if (_log.shouldLog(Log.INFO))
                 _log.info(getJobId() + ": Key search expired");
-            _state.complete(true);
+            _state.complete();
             fail();
         } else if (_state.getAttempted().size() > MAX_PEERS_QUERIED) {
             if (_log.shouldLog(Log.INFO))
                 _log.info(getJobId() + ": Too many peers quried");
-            _state.complete(true);
+            _state.complete();
             fail();
         } else {
             //_log.debug("Continuing search");
@@ -247,19 +272,20 @@ class SearchJob extends JobImpl {
             return;
         } 
         int sent = 0;
-        Set attempted = _state.getAttempted();
+        Set<Hash> attempted = _state.getAttempted();
         while (sent <= 0) {
-            boolean onlyFloodfill = onlyQueryFloodfillPeers(getContext());
-            if (_floodfillPeersExhausted && onlyFloodfill && _state.getPending().size() <= 0) {
+            //boolean onlyFloodfill = onlyQueryFloodfillPeers(getContext());
+            boolean onlyFloodfill = true;
+            if (_floodfillPeersExhausted && onlyFloodfill && _state.getPending().isEmpty()) {
                 if (_log.shouldLog(Log.WARN))
                     _log.warn(getJobId() + ": no non-floodfill peers left, and no more pending.  Searched: "
                               + _state.getAttempted().size() + " failed: " + _state.getFailed().size());
                 fail();
                 return;
             }
-            List closestHashes = getClosestRouters(_state.getTarget(), toCheck, attempted);
-            if ( (closestHashes == null) || (closestHashes.size() <= 0) ) {
-                if (_state.getPending().size() <= 0) {
+            List<Hash> closestHashes = getClosestRouters(_state.getTarget(), toCheck, attempted);
+            if ( (closestHashes == null) || (closestHashes.isEmpty()) ) {
+                if (_state.getPending().isEmpty()) {
                     // we tried to find some peers, but there weren't any and no one else is going to answer
                     if (_log.shouldLog(Log.INFO))
                         _log.info(getJobId() + ": No peers left, and none pending!  Already searched: " 
@@ -276,17 +302,16 @@ class SearchJob extends JobImpl {
                 return;
             } else {
                 attempted.addAll(closestHashes);
-                for (Iterator iter = closestHashes.iterator(); iter.hasNext(); ) {
-                    Hash peer = (Hash)iter.next();
-                    DataStructure ds = _facade.getDataStore().get(peer);
+                for (Hash peer : closestHashes) {
+                    DatabaseEntry ds = _facade.getDataStore().get(peer);
                     if (ds == null) {
                         if (_log.shouldLog(Log.INFO))
                             _log.info("Next closest peer " + peer + " was only recently referred to us, sending a search for them");
                         getContext().netDb().lookupRouterInfo(peer, null, null, _timeoutMs);
-                    } else if (!(ds instanceof RouterInfo)) {
+                    } else if (!(ds.getType() == DatabaseEntry.KEY_TYPE_ROUTERINFO)) {
                         if (_log.shouldLog(Log.WARN))
                             _log.warn(getJobId() + ": Error selecting closest hash that wasnt a router! " 
-                                      + peer + " : " + (ds == null ? "null" : ds.getClass().getName()));
+                                      + peer + " : " + ds.getClass().getName());
                         _state.replyTimeout(peer);
                     } else {
                         RouterInfo ri = (RouterInfo)ds;
@@ -295,8 +320,8 @@ class SearchJob extends JobImpl {
                             if (onlyFloodfill)
                                 continue;
 			}
-                        if (ri.isHidden()) {// || // allow querying shitlisted, since its indirect
-                            //getContext().shitlist().isShitlisted(peer)) {
+                        if (ri.isHidden()) {// || // allow querying banlisted, since its indirect
+                            //getContext().banlist().isBanlisted(peer)) {
                             // dont bother
                         } else {
                             _state.addPending(peer);
@@ -328,6 +353,7 @@ class SearchJob extends JobImpl {
         else
             requeuePending(REQUEUE_DELAY);
     }
+
     private void requeuePending(long ms) {
         if (_pendingRequeueJob == null)
             _pendingRequeueJob = new RequeuePending(getContext());
@@ -351,7 +377,7 @@ class SearchJob extends JobImpl {
      *
      * @return ordered list of Hash objects
      */
-    private List getClosestRouters(Hash key, int numClosest, Set alreadyChecked) {
+    private List<Hash> getClosestRouters(Hash key, int numClosest, Set<Hash> alreadyChecked) {
         Hash rkey = getContext().routingKeyGenerator().getRoutingKey(key);
         if (_log.shouldLog(Log.DEBUG))
             _log.debug(getJobId() + ": Current routing key for " + key + ": " + rkey);
@@ -370,14 +396,21 @@ class SearchJob extends JobImpl {
             return;
         } else {
             if (_log.shouldLog(Log.INFO))
-                _log.info(getJobId() + ": Send search to " + router.getIdentity().getHash().toBase64()
-                          + " for " + _state.getTarget().toBase64() 
+                _log.info(getJobId() + ": Send search to " + router.getIdentity().getHash()
+                          + " for " + _state.getTarget()
                           + " w/ timeout " + getPerPeerTimeoutMs(router.getIdentity().calculateHash()));
         }
 
-        getContext().statManager().addRateData("netDb.searchMessageCount", 1, 0);
+        getContext().statManager().addRateData("netDb.searchMessageCount", 1);
 
-        if (_isLease || true) // always send searches out tunnels
+        // To minimize connection congestion, send RI lokups through exploratory tunnels if not connected.
+        // To minimize crypto overhead and response latency, send RI lookups directly if connected.
+        // But not too likely since we don't explore when we're floodfill.
+        // Always send LS lookups thru expl tunnels.
+        // But this is never used for LSes...
+
+        if (_isLease ||
+             !getContext().commSystem().isEstablished(router.getIdentity().calculateHash()))
             sendLeaseSearch(router);
         else
             sendRouterSearch(router);
@@ -390,32 +423,37 @@ class SearchJob extends JobImpl {
      *
      */
     protected void sendLeaseSearch(RouterInfo router) {
-        TunnelInfo inTunnel = getInboundTunnelId(); 
+        Hash to = router.getIdentity().getHash();
+        TunnelInfo inTunnel = getContext().tunnelManager().selectInboundExploratoryTunnel(to);
         if (inTunnel == null) {
-            _log.warn("No tunnels to get search replies through!  wtf!");
+            _log.warn("No tunnels to get search replies through!");
             getContext().jobQueue().addJob(new FailedJob(getContext(), router));
             return;
         }
         TunnelId inTunnelId = inTunnel.getReceiveTunnelId(0);
 
-        // this will fail if we've shitlisted our inbound gateway, but the gw may not necessarily
-        // be shitlisted by whomever needs to contact them, so we don't need to check this
+        // this will fail if we've banlisted our inbound gateway, but the gw may not necessarily
+        // be banlisted by whomever needs to contact them, so we don't need to check this
         
         //RouterInfo inGateway = getContext().netDb().lookupRouterInfoLocally(inTunnel.getPeer(0));
         //if (inGateway == null) {
-        //    _log.error("We can't find the gateway to our inbound tunnel?! wtf");
+        //    _log.error("We can't find the gateway to our inbound tunnel?!");
         //    getContext().jobQueue().addJob(new FailedJob(getContext(), router));
         //    return;
         //}
 	
-        int timeout = getPerPeerTimeoutMs(router.getIdentity().getHash());
+        int timeout = getPerPeerTimeoutMs(to);
         long expiration = getContext().clock().now() + timeout;
 
-        DatabaseLookupMessage msg = buildMessage(inTunnelId, inTunnel.getPeer(0), expiration);	
-	
-        TunnelInfo outTunnel = getOutboundTunnelId();
+        I2NPMessage msg = buildMessage(inTunnelId, inTunnel.getPeer(0), expiration, router);	
+        if (msg == null) {
+            getContext().jobQueue().addJob(new FailedJob(getContext(), router));
+            return;
+        }
+
+        TunnelInfo outTunnel = getContext().tunnelManager().selectOutboundExploratoryTunnel(to);
         if (outTunnel == null) {
-            _log.warn("No tunnels to send search out through!  wtf!");
+            _log.warn("No tunnels to send search out through! Impossible?");
             getContext().jobQueue().addJob(new FailedJob(getContext(), router));
             return;
         }        
@@ -423,10 +461,10 @@ class SearchJob extends JobImpl {
 
 	
         if (_log.shouldLog(Log.DEBUG))
-            _log.debug(getJobId() + ": Sending leaseSet search to " + router.getIdentity().getHash().toBase64() 
-                       + " for " + msg.getSearchKey().toBase64() + " w/ replies through [" 
-                       + msg.getFrom().toBase64() + "] via tunnel [" 
-                       + msg.getReplyTunnel() + "]");
+            _log.debug(getJobId() + ": Sending search to " + to
+                       + " for " + getState().getTarget() + " w/ replies through " 
+                       + inTunnel.getPeer(0) + " via tunnel " 
+                       + inTunnelId);
 
         SearchMessageSelector sel = new SearchMessageSelector(getContext(), router, _expiration, _state);
         SearchUpdateReplyFoundJob reply = new SearchUpdateReplyFoundJob(getContext(), router, _state, _facade, 
@@ -434,8 +472,9 @@ class SearchJob extends JobImpl {
         
         if (FloodfillNetworkDatabaseFacade.isFloodfill(router))
             _floodfillSearchesOutstanding++;
-        getContext().messageRegistry().registerPending(sel, reply, new FailedJob(getContext(), router), timeout);
-        getContext().tunnelDispatcher().dispatchOutbound(msg, outTunnelId, router.getIdentity().getHash());
+        getContext().messageRegistry().registerPending(sel, reply, new FailedJob(getContext(), router));
+        // TODO pass a priority to the dispatcher
+        getContext().tunnelDispatcher().dispatchOutbound(msg, outTunnelId, to);
     }
     
     /** we're searching for a router, so we can just send direct */
@@ -443,39 +482,28 @@ class SearchJob extends JobImpl {
         int timeout = _facade.getPeerTimeout(router.getIdentity().getHash());
         long expiration = getContext().clock().now() + timeout;
 
-        DatabaseLookupMessage msg = buildMessage(expiration);
+        // use the 4-arg one so we pick up the override in ExploreJob
+        //I2NPMessage msg = buildMessage(expiration);
+        I2NPMessage msg = buildMessage(null, router.getIdentity().getHash(), expiration, router);	
+        if (msg == null) {
+            getContext().jobQueue().addJob(new FailedJob(getContext(), router));
+            return;
+        }
 
         if (_log.shouldLog(Log.DEBUG))
-            _log.debug(getJobId() + ": Sending router search to " + router.getIdentity().getHash().toBase64() 
-                      + " for " + msg.getSearchKey().toBase64() + " w/ replies to us [" 
-                      + msg.getFrom().toBase64() + "]");
+            _log.debug(getJobId() + ": Sending router search directly to " + router.getIdentity().getHash()
+                      + " for " + _state.getTarget());
         SearchMessageSelector sel = new SearchMessageSelector(getContext(), router, _expiration, _state);
         SearchUpdateReplyFoundJob reply = new SearchUpdateReplyFoundJob(getContext(), router, _state, _facade, this);
         SendMessageDirectJob j = new SendMessageDirectJob(getContext(), msg, router.getIdentity().getHash(), 
-                                                          reply, new FailedJob(getContext(), router), sel, timeout, SEARCH_PRIORITY);
+                                                          reply, new FailedJob(getContext(), router), sel, timeout,
+                                                          OutNetMessage.PRIORITY_EXPLORATORY);
         if (FloodfillNetworkDatabaseFacade.isFloodfill(router))
             _floodfillSearchesOutstanding++;
         j.runJob();
         //getContext().jobQueue().addJob(j);
     }
     
-    /** 
-     * what tunnel will we send the search out through? 
-     *
-     * @return tunnel id (or null if none are found)
-     */
-    private TunnelInfo getOutboundTunnelId() {
-        return getContext().tunnelManager().selectOutboundTunnel();
-    }
-    
-    /**
-     * what tunnel will we get replies through?
-     *
-     * @return tunnel id (or null if none are found)
-     */
-    private TunnelInfo getInboundTunnelId() {
-        return getContext().tunnelManager().selectInboundTunnel();
-    }
 
     /**
      * Build the database search message 
@@ -483,8 +511,13 @@ class SearchJob extends JobImpl {
      * @param replyTunnelId tunnel to receive replies through
      * @param replyGateway gateway for the reply tunnel
      * @param expiration when the search should stop 
+     * @param peer unused here; see ExploreJob extension
+     *
+     * @return a DatabaseLookupMessage
      */
-    protected DatabaseLookupMessage buildMessage(TunnelId replyTunnelId, Hash replyGateway, long expiration) {
+    protected I2NPMessage buildMessage(TunnelId replyTunnelId, Hash replyGateway, long expiration, RouterInfo peer) {
+        throw new UnsupportedOperationException("see ExploreJob");
+/*******
         DatabaseLookupMessage msg = new DatabaseLookupMessage(getContext(), true);
         msg.setSearchKey(_state.getTarget());
         //msg.setFrom(replyGateway.getIdentity().getHash());
@@ -493,6 +526,7 @@ class SearchJob extends JobImpl {
         msg.setMessageExpiration(expiration);
         msg.setReplyTunnel(replyTunnelId);
         return msg;
+*********/
     }
     
     /**
@@ -500,6 +534,7 @@ class SearchJob extends JobImpl {
      * replies sent back to us directly)
      *
      */
+/******* always send through the lease
     protected DatabaseLookupMessage buildMessage(long expiration) {
         DatabaseLookupMessage msg = new DatabaseLookupMessage(getContext(), true);
         msg.setSearchKey(_state.getTarget());
@@ -509,11 +544,13 @@ class SearchJob extends JobImpl {
         msg.setReplyTunnel(null);
         return msg;
     }
+*********/
     
+    /** found a reply */
     void replyFound(DatabaseSearchReplyMessage message, Hash peer) {
         long duration = _state.replyFound(peer);
         // this processing can take a while, so split 'er up
-        getContext().jobQueue().addJob(new SearchReplyJob(getContext(), this, (DatabaseSearchReplyMessage)message, peer, duration));
+        getContext().jobQueue().addJob(new SearchReplyJob(getContext(), this, message, peer, duration));
     }
     
     /**
@@ -557,13 +594,13 @@ class SearchJob extends JobImpl {
             _state.replyTimeout(_peer);
             if (_penalizePeer) { 
                 if (_log.shouldLog(Log.INFO))
-                    _log.info("Penalizing peer for timeout on search: " + _peer.toBase64() + " after " + (getContext().clock().now() - _sentOn));
+                    _log.info("Penalizing peer for timeout on search: " + _peer + " after " + (getContext().clock().now() - _sentOn));
                 getContext().profileManager().dbLookupFailed(_peer);
             } else {
                 if (_log.shouldLog(Log.ERROR))
-                    _log.error("NOT (!!) Penalizing peer for timeout on search: " + _peer.toBase64());
+                    _log.error("NOT (!!) Penalizing peer for timeout on search: " + _peer);
             }
-            getContext().statManager().addRateData("netDb.failedPeers", 1, 0);
+            getContext().statManager().addRateData("netDb.failedPeers", 1);
             searchNext();
         }
         public String getName() { return "Kademlia Search Failed"; }
@@ -581,7 +618,7 @@ class SearchJob extends JobImpl {
 	
         if (_keepStats) {
             long time = getContext().clock().now() - _state.getWhenStarted();
-            getContext().statManager().addRateData("netDb.successTime", time, 0);
+            getContext().statManager().addRateData("netDb.successTime", time);
             getContext().statManager().addRateData("netDb.successPeers", _state.getAttempted().size(), time);
         }
         if (_onSuccess != null)
@@ -616,7 +653,7 @@ class SearchJob extends JobImpl {
      *
      */
     private void resend() {
-        DataStructure ds = _facade.lookupLeaseSetLocally(_state.getTarget());
+        DatabaseEntry ds = _facade.lookupLeaseSetLocally(_state.getTarget());
         if (ds == null) {
             if (SHOULD_RESEND_ROUTERINFO) {
                 ds = _facade.lookupRouterInfoLocally(_state.getTarget());
@@ -624,11 +661,10 @@ class SearchJob extends JobImpl {
                     _facade.sendStore(_state.getTarget(), ds, null, null, RESEND_TIMEOUT, _state.getSuccessful());
             }
         } else {
-            Set sendTo = _state.getRepliedPeers(); // _state.getFailed();
+            Set<Hash> sendTo = _state.getRepliedPeers(); // _state.getFailed();
             sendTo.addAll(_state.getPending());
             int numSent = 0;
-            for (Iterator iter = sendTo.iterator(); iter.hasNext(); ) {
-                Hash peer = (Hash)iter.next();
+            for (Hash peer : sendTo) {
                 RouterInfo peerInfo = _facade.lookupRouterInfoLocally(peer);
                 if (peerInfo == null) continue;
                 if (resend(peerInfo, (LeaseSet)ds))
@@ -645,20 +681,18 @@ class SearchJob extends JobImpl {
      * provide us with the data when we asked them.  
      */
     private boolean resend(RouterInfo toPeer, LeaseSet ls) {
+        Hash to = toPeer.getIdentity().getHash();
         DatabaseStoreMessage msg = new DatabaseStoreMessage(getContext());
-        msg.setKey(ls.getDestination().calculateHash());
-        msg.setLeaseSet(ls);
+        msg.setEntry(ls);
         msg.setMessageExpiration(getContext().clock().now() + RESEND_TIMEOUT);
 
-        TunnelInfo outTunnel = getContext().tunnelManager().selectOutboundTunnel();
+        TunnelInfo outTunnel = getContext().tunnelManager().selectOutboundExploratoryTunnel(to);
 
         if (outTunnel != null) {
-            TunnelId targetTunnelId = null; // not needed
-            Job onSend = null; // not wanted
-            
             if (_log.shouldLog(Log.DEBUG))
-                _log.debug("resending leaseSet out to " + toPeer.getIdentity().getHash() + " through " + outTunnel + ": " + msg);
-            getContext().tunnelDispatcher().dispatchOutbound(msg, outTunnel.getSendTunnelId(0), null, toPeer.getIdentity().getHash());
+                _log.debug("resending leaseSet out to " + to + " through " + outTunnel + ": " + msg);
+            // TODO pass a priority to the dispatcher
+            getContext().tunnelDispatcher().dispatchOutbound(msg, outTunnel.getSendTunnelId(0), null, to);
             return true;
         } else {
             if (_log.shouldLog(Log.WARN))
@@ -673,7 +707,7 @@ class SearchJob extends JobImpl {
     protected void fail() {
         if (isLocal()) {
             if (_log.shouldLog(Log.ERROR))
-                _log.error(getJobId() + ": why did we fail if the target is local?: " + _state.getTarget().toBase64(), new Exception("failure cause"));
+                _log.error(getJobId() + ": why did we fail if the target is local?: " + _state.getTarget(), new Exception("failure cause"));
             succeed();
             return;
         }
@@ -688,7 +722,7 @@ class SearchJob extends JobImpl {
         getContext().statManager().addRateData("netDb.failedAttemptedPeers", attempted, time);
         
         if (_keepStats) {
-            getContext().statManager().addRateData("netDb.failedTime", time, 0);
+            getContext().statManager().addRateData("netDb.failedTime", time);
             //_facade.fail(_state.getTarget());
         }
         if (_onFailure != null)
@@ -725,10 +759,10 @@ class SearchJob extends JobImpl {
     }
     
     private void handleDeferred(boolean success) {
-        List deferred = null;
+        List<Search> deferred = null;
         synchronized (_deferredSearches) {
-            if (_deferredSearches.size() > 0) {
-                deferred = new ArrayList(_deferredSearches);
+            if (!_deferredSearches.isEmpty()) {
+                deferred = new ArrayList<Search>(_deferredSearches);
                 _deferredSearches.clear();
             }
             _deferredCleared = true;
@@ -736,7 +770,7 @@ class SearchJob extends JobImpl {
         if (deferred != null) {
             long now = getContext().clock().now();
             for (int i = 0; i < deferred.size(); i++) {
-                Search cur = (Search)deferred.get(i);
+                Search cur = deferred.get(i);
                 if (cur.getExpiration() < now)
                     getContext().jobQueue().addJob(cur.getOnFail());
                 else if (success)
@@ -747,11 +781,11 @@ class SearchJob extends JobImpl {
         }
     }
     
-    private class Search {
-        private Job _onFind;
-        private Job _onFail;
-        private long _expiration;
-        private boolean _isLease;
+    private static class Search {
+        private final Job _onFind;
+        private final Job _onFail;
+        private final long _expiration;
+        private final boolean _isLease;
         
         public Search(Job onFind, Job onFail, long expiration, boolean isLease) {
             _onFind = onFind;
@@ -762,156 +796,31 @@ class SearchJob extends JobImpl {
         public Job getOnFind() { return _onFind; }
         public Job getOnFail() { return _onFail; }
         public long getExpiration() { return _expiration; }
-        public boolean getIsLease() { return _isLease; }
     }
     
     public String getName() { return "Kademlia NetDb Search"; }
     
+    @Override
     public String toString() { 
         return super.toString() + " started " 
                + DataHelper.formatDuration((getContext().clock().now() - _startedOn)) + " ago";
     }
     
     boolean wasAttempted(Hash peer) { return _state.wasAttempted(peer); }
+
     long timeoutMs() { return _timeoutMs; }
-    boolean add(Hash peer) { return _facade.getKBuckets().add(peer); }
+
+    /** @return true if peer was new */
+    boolean add(Hash peer) {
+        boolean rv = _facade.getKBuckets().add(peer);
+        if (rv) {
+            if (_log.shouldLog(Log.DEBUG))
+                _log.debug(getJobId() + ": Queueing up for next time: " + peer);
+            Set<Hash> s = Collections.singleton(peer);
+            _facade.queueForExploration(s);
+        }
+        return rv;
+    }
+
     void decrementOutstandingFloodfillSearches() { _floodfillSearchesOutstanding--; }
 }
-
-class SearchReplyJob extends JobImpl {
-    private DatabaseSearchReplyMessage _msg;
-    private Log _log;
-    /** 
-     * Peer who we think sent us the reply.  Note: could be spoofed!  If the
-     * attacker knew we were searching for a particular key from a 
-     * particular peer, they could send us some searchReply messages with
-     * shitty values, trying to get us to consider that peer unreliable.  
-     * Potential fixes include either authenticated 'from' address or use a
-     * nonce in the search + searchReply (and check for it in the selector).
-     *
-     */
-    private Hash _peer;
-    private int _curIndex;
-    private int _invalidPeers;
-    private int _seenPeers;
-    private int _newPeers;
-    private int _duplicatePeers;
-    private int _repliesPendingVerification;
-    private long _duration;
-    private SearchJob _searchJob;
-    public SearchReplyJob(RouterContext enclosingContext, SearchJob job, DatabaseSearchReplyMessage message, Hash peer, long duration) {
-        super(enclosingContext);
-        _log = enclosingContext.logManager().getLog(getClass());
-        _searchJob = job;
-        _msg = message;
-        _peer = peer;
-        _curIndex = 0;
-        _invalidPeers = 0;
-        _seenPeers = 0;
-        _newPeers = 0;
-        _duplicatePeers = 0;
-        _repliesPendingVerification = 0;
-    }
-    public String getName() { return "Process Reply for Kademlia Search"; }
-    public void runJob() {
-        if (_curIndex >= _msg.getNumReplies()) {
-            if (_repliesPendingVerification > 0) {
-                // we received new references from the peer, but still 
-                // haven't verified all of them, so lets give it more time
-                requeue(_searchJob.timeoutMs());
-            } else {
-                // either they didn't tell us anything new or we have verified
-                // (or failed to verify) all of them.  we're done
-                getContext().profileManager().dbLookupReply(_peer, _newPeers, _seenPeers, 
-                                                           _invalidPeers, _duplicatePeers, _duration);
-                if (_newPeers > 0)
-                    _searchJob.newPeersFound(_newPeers);
-            }
-        } else {
-            Hash peer = _msg.getReply(_curIndex);
-
-            boolean shouldAdd = false;
-
-            RouterInfo info = getContext().netDb().lookupRouterInfoLocally(peer);
-            if (info == null) {
-                // if the peer is giving us lots of bad peer references, 
-                // dont try to fetch them.
-
-                boolean sendsBadInfo = getContext().profileOrganizer().peerSendsBadReplies(_peer);
-                if (!sendsBadInfo) {
-                    // we don't need to search for everthing we're given here - only ones that
-                    // are next in our search path...
-                    // note: no need to think about shitlisted targets in the netdb search, given
-                    //       the floodfill's behavior
-                    //if (getContext().shitlist().isShitlisted(peer)) {
-                    //    if (_log.shouldLog(Log.INFO))
-                    //        _log.info("Not looking for a shitlisted peer...");
-                    //    getContext().statManager().addRateData("netDb.searchReplyValidationSkipped", 1, 0);
-                    //} else {
-                        //getContext().netDb().lookupRouterInfo(peer, new ReplyVerifiedJob(getContext(), peer), new ReplyNotVerifiedJob(getContext(), peer), _timeoutMs);
-                        //_repliesPendingVerification++;
-                        shouldAdd = true;
-                    //}
-                } else {
-                    if (_log.shouldLog(Log.INFO))
-                        _log.info("Peer " + _peer.toBase64() + " sends us bad replies, so not verifying " + peer.toBase64());
-                    getContext().statManager().addRateData("netDb.searchReplyValidationSkipped", 1, 0);
-                }
-            }
-
-            if (_searchJob.wasAttempted(peer)) {
-                _duplicatePeers++;
-            } 
-            if (_log.shouldLog(Log.DEBUG))
-                _log.debug(getJobId() + ": dbSearchReply received on search referencing router " + peer);
-            if (shouldAdd) {
-                if (_searchJob.add(peer))
-                    _newPeers++;
-                else
-                    _seenPeers++;
-            }
-
-            _curIndex++;
-            requeue(0);
-        }
-    }
-    void replyVerified() {
-        if (_log.shouldLog(Log.INFO))
-            _log.info("Peer reply from " + _peer.toBase64());
-        _repliesPendingVerification--;
-        getContext().statManager().addRateData("netDb.searchReplyValidated", 1, 0);
-    }
-    void replyNotVerified() {
-        if (_log.shouldLog(Log.INFO))
-            _log.info("Peer reply from " + _peer.toBase64());
-        _repliesPendingVerification--;
-        _invalidPeers++;
-        getContext().statManager().addRateData("netDb.searchReplyNotValidated", 1, 0);
-    }
-}
-
-/** the peer gave us a reference to a new router, and we were able to fetch it */
-class ReplyVerifiedJob extends JobImpl {
-    private Hash _key;
-    private SearchReplyJob _replyJob;
-    public ReplyVerifiedJob(RouterContext enclosingContext, SearchReplyJob srj, Hash key) {
-        super(enclosingContext);
-        _replyJob = srj;
-        _key = key;
-    }
-    public String getName() { return "Search reply value verified"; }
-    public void runJob() { _replyJob.replyVerified(); }
-}
-
-/** the peer gave us a reference to a new router, and we were NOT able to fetch it */
-class ReplyNotVerifiedJob extends JobImpl {
-    private Hash _key;
-    private SearchReplyJob _replyJob;
-    public ReplyNotVerifiedJob(RouterContext enclosingContext, SearchReplyJob srj, Hash key) {
-        super(enclosingContext);
-        _key = key;
-        _replyJob = srj;
-    }
-    public String getName() { return "Search reply value NOT verified"; }
-    public void runJob() { _replyJob.replyNotVerified(); }
-}
