@@ -39,24 +39,25 @@ class StoreMessageSelector implements MessageSelector {
     public long getExpiration() { return _expiration; }
 
     public boolean isMatch(I2NPMessage message) {
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug(_storeJobId + ": isMatch("+message.getClass().getName() + ") [want deliveryStatusMessage from " 
-                       + _peer + "]");
-        if (message instanceof DeliveryStatusMessage) {
+        //if (_log.shouldDebug())
+        //    _log.debug(_storeJobId + ": isMatch(" + message.getClass().getSimpleName() + ") [want DSM from " 
+        //               + _peer + ']');
+        if (message.getType() == DeliveryStatusMessage.MESSAGE_TYPE) {
             DeliveryStatusMessage msg = (DeliveryStatusMessage)message;
             if (msg.getMessageId() == _waitingForId) {
-                if (_log.shouldLog(Log.INFO))
+                if (_log.shouldInfo())
                     _log.info(_storeJobId + ": Found match for the key we're waiting for: " + _waitingForId);
                 _found = true;
                 return true;
             } else {
-                if (_log.shouldLog(Log.DEBUG))
-                    _log.debug(_storeJobId + ": DeliveryStatusMessage of a key we're not looking for");
+                //if (_log.shouldDebug())
+                //    _log.debug(_storeJobId + ": DeliveryStatusMessage of " + msg.getMessageId() +
+                //               " but waiting for " + _waitingForId);
                 return false;
             }
         } else {
-            if (_log.shouldLog(Log.DEBUG))
-                _log.debug(_storeJobId + ": Not a DeliveryStatusMessage");
+            //if (_log.shouldLog(Log.DEBUG))
+            //    _log.debug(_storeJobId + ": Not a DeliveryStatusMessage");
             return false;
         }
     }
