@@ -169,6 +169,9 @@ public class GeoIP {
                     add(ip);
                 }
                 String lastIP = _context.getProperty(UDPTransport.PROP_IP);
+                if (lastIP != null)
+                    add(lastIP);
+                lastIP = _context.getProperty(UDPTransport.PROP_IPV6);
                 if (lastIP != null)
                     add(lastIP);
                 // IPv4
@@ -512,8 +515,14 @@ public class GeoIP {
             }
             if (country == null) {
                 String lastIP = _context.getProperty(UDPTransport.PROP_IP);
-                if (lastIP != null)
+                if (lastIP != null) {
                     country = get(lastIP);
+                    if (country == null) {
+                        lastIP = _context.getProperty(UDPTransport.PROP_IPV6);
+                        if (lastIP != null)
+                            country = get(lastIP);
+                    }
+                }
             }
         }
         if (_log.shouldInfo())
