@@ -9,9 +9,19 @@ package net.i2p.router.transport;
  */
 
 import net.i2p.data.Hash;
-import net.i2p.data.RouterIdentity;
+import net.i2p.data.router.RouterIdentity;
 import net.i2p.data.i2np.I2NPMessage;
 
 public interface TransportEventListener {
+
+    /**
+     * Message received
+     *
+     * @param message non-null
+     * @param fromRouter may be null
+     * @param fromRouterHash may be null, calculated from fromRouter if null
+     */
     public void messageReceived(I2NPMessage message, RouterIdentity fromRouter, Hash fromRouterHash);
+
+    public void transportAddressChanged();
 }
