@@ -34,6 +34,8 @@ class UPnPManager {
     private final RouterContext _context;
     private final UPnP _upnp;
     private final UPnPCallback _upnpCallback;
+    private final UPnPScannerCallback _scannerCallback;
+    private final DelayedCallback _delayedCallback;
     private volatile boolean _isRunning;
     private volatile boolean _shouldBeRunning;
     private volatile long _lastRescan;
@@ -73,6 +75,8 @@ class UPnPManager {
         Debug.initialize(context);
         _upnp = new UPnP(context);
         _upnpCallback = new UPnPCallback();
+        _scannerCallback = _context.router().getUPnPScannerCallback();
+        _delayedCallback = (_scannerCallback != null) ? new DelayedCallback() : null;
         _rescanner = new Rescanner();
     }
     
@@ -91,7 +95,16 @@ class UPnPManager {
                 // and will eventually hit 1024 and then negative
                 _upnp.setHTTPPort(_context.getProperty(PROP_HTTP_PORT, DEFAULT_HTTP_PORT));
                 _upnp.setSSDPPort(_context.getProperty(PROP_SSDP_PORT, DEFAULT_SSDP_PORT));
+                if (_scannerCallback != null) {
+                    _scannerCallback.beforeScan();
+                }
                 _isRunning = _upnp.runPlugin();
+                if (_scannerCallback != null) {
+                    if (_isRunning)
+                        _delayedCallback.reschedule();
+                    else
+                        _scannerCallback.afterScan();
+                }
                 if (_log.shouldDebug())
                     _log.info("UPnP runPlugin took " + (_context.clock().now() - b));
             } catch (RuntimeException e) {
@@ -100,6 +113,9 @@ class UPnPManager {
                     _log.error("UPnP error, please report", e);
                     _errorLogged = true;
                 }
+                if (_scannerCallback != null) {
+                    _scannerCallback.afterScan();
+                }
             }
         }
         if (_isRunning) {
@@ -143,18 +159,21 @@ class UPnPManager {
      *  Should be fast. This only starts the search, the responses
      *  will come in over the MX time (3 seconds).
      *
+     *  @return true if a rescan was actually fired off
      *  @since 0.9.18
      */
-    public synchronized void rescan() {
+    public synchronized boolean rescan() {
         if (!_shouldBeRunning)
-            return;
+            return false;
         if (_context.router().gracefulShutdownInProgress())
-            return;
+            return false;
         long now = System.currentTimeMillis();
         if (_lastRescan + RESCAN_MIN_DELAY > now)
-            return;
+            return false;
         _lastRescan = now;
         if (_isRunning) {
+            if (_scannerCallback != null)
+                _scannerCallback.beforeScan();
             if (_log.shouldLog(Log.DEBUG))
                 _log.debug("UPnP Rescan");
             // TODO default search MX (jitter) is 3 seconds... reduce?
@@ -162,9 +181,12 @@ class UPnPManager {
             // Adaptive Jitter Control for UPnP M-Search
             // Kevin Mills and Christopher Dabrowski
             _upnp.search();
+            if (_scannerCallback != null)
+                _delayedCallback.reschedule();
         } else {
             start();
         }
+        return true;
     }
 
     /**
@@ -186,6 +208,33 @@ class UPnPManager {
             }
         }
     }
+
+    /**
+     * Delayed Callback
+     *
+     * @since 0.9.41
+     */
+    private class DelayedCallback extends SimpleTimer2.TimedEvent {
+
+        /** caller must reschedule() */
+        public DelayedCallback() {
+            super(_context.simpleTimer2());
+        }
+
+        public void timeReached() {
+             _scannerCallback.afterScan();
+        }
+
+        /**
+         *  Pushes out.
+         *  We do it this way because we may have two scans running concurrently,
+         *  we only want to call afterScan() once.
+         */
+        void reschedule() {
+            // false == use latest time
+            reschedule((_upnp.getSearchMx() * 1000) + 500, false);
+        }
+    }
     
     /**
      * Call when the ports might have changed
@@ -195,7 +244,7 @@ class UPnPManager {
      */
     public void update(Set<TransportManager.Port> ports) {
         if (_log.shouldLog(Log.DEBUG))
-            _log.debug("UPnP Update with " + ports.size() + " ports");
+            _log.debug("UPnP Update with " + ports.size() + " ports", new Exception("I did it"));
 
         //synchronized(this) {
             // TODO
@@ -240,7 +289,7 @@ class UPnPManager {
         /** Called to indicate status on one or more forwarded ports. */
         public void portForwardStatus(Map<ForwardPort,ForwardPortStatus> statuses) {
             if (_log.shouldLog(Log.DEBUG))
-                 _log.debug("UPnP Callback:");
+                 _log.debug("UPnP Callback: with " + statuses.size() + " statuses");
             // Let's not have two of these running at once.
             // Deadlock reported in ticket #1699
             // and the locking isn't foolproof in UDPTransport.
@@ -254,7 +303,7 @@ class UPnPManager {
         private void locked_PFS(Map<ForwardPort,ForwardPortStatus> statuses) {
             byte[] ipaddr = null;
             DetectedIP[] ips = _upnp.getAddress();
-            if (ips != null) {
+            if (ips != null && ips.length > 0) {
                 for (DetectedIP ip : ips) {
                     // store the first public one and tell the transport manager if it changed
                     // Note that getAddress() will actually return a max of one address.
@@ -268,6 +317,9 @@ class UPnPManager {
                         }
                         ipaddr = ip.publicAddress.getAddress();
                         break;
+                    } else {
+                        if (_log.shouldWarn())
+                            _log.warn("Unusable external address: " + ip.publicAddress + " type: " + ip.natType);
                     }
                 }
             } else {
@@ -275,19 +327,28 @@ class UPnPManager {
                     _log.debug("No external address returned");
             }
 
+            if (statuses.isEmpty()) {
+                if (_log.shouldWarn())
+                    _log.warn("No statuses returned");
+                return;
+            }
+
             for (Map.Entry<ForwardPort, ForwardPortStatus> entry : statuses.entrySet()) {
                 ForwardPort fp = entry.getKey();
                 ForwardPortStatus fps = entry.getValue();
-                if (_log.shouldLog(Log.DEBUG))
-                    _log.debug(fp.name + " " + fp.protocol + " " + fp.portNumber +
+                if (_log.shouldDebug())
+                    _log.debug("FPS: " + fp.name + ' ' + fp.protocol + ' ' + fp.portNumber +
                                " status: " + fps.status + " reason: " + fps.reasonString + " ext port: " + fps.externalPort);
                 String style;
-                if (fp.protocol == ForwardPort.PROTOCOL_UDP_IPV4)
+                if (fp.protocol == ForwardPort.PROTOCOL_UDP_IPV4) {
                     style = "SSU";
-                else if (fp.protocol == ForwardPort.PROTOCOL_TCP_IPV4)
+                } else if (fp.protocol == ForwardPort.PROTOCOL_TCP_IPV4) {
                     style = "NTCP";
-                else
+                } else {
+                    if (_log.shouldWarn())
+                        _log.debug("Unknown protocol " + fp.protocol);
                     continue;
+                }
                 boolean success = fps.status >= ForwardPortStatus.MAYBE_SUCCESS;
                 // deadlock path 2
                 _manager.forwardPortStatus(style, ipaddr, fp.portNumber, fps.externalPort, success, fps.reasonString);
