@@ -4,6 +4,7 @@ import java.util.concurrent.LinkedBlockingQueue;
 
 import net.i2p.I2PAppContext;
 import net.i2p.crypto.EncType;
+import net.i2p.crypto.KeyFactory;
 import net.i2p.crypto.KeyPair;
 import net.i2p.data.PrivateKey;
 import net.i2p.data.PublicKey;
@@ -19,7 +20,7 @@ import net.i2p.util.SystemVersion;
  *
  *  @since 0.9.36 from DHSessionKeyFactory.PrecalcRunner
  */
-public class X25519KeyFactory extends I2PThread {
+public class X25519KeyFactory extends I2PThread implements KeyFactory {
 
     private final I2PAppContext _context;
     private final Log _log;
@@ -151,8 +152,10 @@ public class X25519KeyFactory extends I2PThread {
      * to be put back onto the queue for reuse.
      */
     public void returnUnused(KeyPair kp) {
+/*
         _context.statManager().addRateData("crypto.XDHReused", 1);
         _keys.offer(kp);
+*/
     }
 
     /** @return true if successful, false if full */
