@@ -872,9 +872,12 @@ public class NTCPTransport extends TransportImpl {
                     props.setProperty(RouterAddress.PROP_HOST, ia.getHostAddress());
                     props.setProperty(RouterAddress.PROP_PORT, Integer.toString(port));
                     addNTCP2Options(props);
-                    int cost = getDefaultCost(ia instanceof Inet6Address);
-                    myAddress = new RouterAddress(getPublishStyle(), props, cost);
-                    replaceAddress(myAddress);
+                    boolean ipv6 = ia instanceof Inet6Address;
+                    if (!ipv6 || !_context.getBooleanProperty(PROP_IPV6_FIREWALLED)) {
+                        int cost = getDefaultCost(ipv6);
+                        myAddress = new RouterAddress(getPublishStyle(), props, cost);
+                        replaceAddress(myAddress);
+                    }
                 }
             } else if (_enableNTCP2) {
                 setOutboundNTCP2Address();
@@ -1674,7 +1677,15 @@ public class NTCPTransport extends TransportImpl {
                 return false;
             }
         }
-        addNTCP2Options(newProps);
+
+        if (!isIPv6 || newProps.containsKey(RouterAddress.PROP_HOST) || getIPv6Config() == IPV6_ONLY) {
+            addNTCP2Options(newProps);
+        } else {
+            // IPv6
+            // We have an IPv4 address, IPv6 transitioned to firewalled,
+            // so just remove the v6 address
+            newAddr = null;
+        }
 
         // stopListening stops the pumper, readers, and writers, so required even if
         // oldAddr == null since startListening starts them all again
