@@ -51,7 +51,7 @@ and currently set to 0.  Peers using a different protocol version will
 not be able to communicate with this peer, though earlier versions not
 using this flag are.</p>
 
-<h2><a name="payload">Payload</a></h2>
+<h2 id="payload">Payload</h2>
 
 <p>Within the AES encrypted payload, there is a minimal common structure
 to the various messages - a one byte flag and a four byte sending 
@@ -1614,12 +1614,19 @@ class PacketBuilder {
         off += totalSize;
         System.arraycopy(iv, 0, data, off, UDPPacket.IV_SIZE);
         off += UDPPacket.IV_SIZE;
-        DataHelper.toLong(data, off, 2, totalSize /* ^ PROTOCOL_VERSION */ );
+        // version is zero, unlikely to ever change
+        int plval = totalSize /* ^ PacketBuilder.PROTOCOL_VERSION */ ;
+        // network ID cross-check, proposal 147
+        int netid = _context.router().getNetworkID();
+        if (netid != 2) {
+            plval ^= (netid - 2) << 8;
+        }
+        DataHelper.toLong(data, off, 2, plval);
         
         int hmacLen = totalSize + UDPPacket.IV_SIZE + 2;
         //Hash hmac = _context.hmac().calculate(macKey, data, hmacOff, hmacLen);
         byte[] ba = SimpleByteCache.acquire(Hash.HASH_LENGTH);
-        _context.hmac().calculate(macKey, data, hmacOff, hmacLen, ba, 0);
+        _transport.getHMAC().calculate(macKey, data, hmacOff, hmacLen, ba, 0);
         
         if (_log.shouldLog(Log.DEBUG))
             _log.debug("Authenticating " + pkt.getLength() +
