@@ -97,12 +97,6 @@ class UDPEndpoint implements SocketListener {
     }
 ********/
     
-    /** 8998 is monotone, and 31000 is the wrapper outbound, so let's stay between those */
-    public static final String PROP_MIN_PORT = "i2np.udp.minPort";
-    public static final String PROP_MAX_PORT = "i2np.udp.maxPort";
-    /** Was 9111, increase to skip Tor browser at 9050 */
-    private static final int MIN_RANDOM_PORT = 9151;
-    private static final int MAX_RANDOM_PORT = 30777;
     private static final int MAX_PORT_RETRIES = 20;
 
     /**
@@ -115,9 +109,7 @@ class UDPEndpoint implements SocketListener {
         DatagramSocket socket = null;
         int port = _listenPort;
         if (port > 0 && !TransportUtil.isValidPort(port)) {
-            _log.error("Specified UDP port " + port + " is not valid, selecting a new port");
-            // See isValidPort() for list
-            _log.error("Invalid ports are: 0-1023, 1900, 2049, 2827, 3659, 4045, 4444, 4445, 6000, 6665-6669, 6697, 7650-7668, 8998, 9001, 9030, 9050, 9100, 9150, 31000, 32000, 65536+");
+            TransportUtil.logInvalidPort(_log, "UDP", port);
             port = -1;
         }
 
@@ -125,7 +117,7 @@ class UDPEndpoint implements SocketListener {
              if (port <= 0) {
                  // try random ports rather than just do new DatagramSocket()
                  // so we stay out of the way of other I2P stuff
-                 port = selectRandomPort(_context);
+                 port = TransportUtil.selectRandomPort(_context, UDPTransport.STYLE);
              }
              try {
                  if (_bindAddress == null)
@@ -151,16 +143,6 @@ class UDPEndpoint implements SocketListener {
         return socket;
     }
 
-    /**
-     *  Pick a random port between the configured boundaries
-     *  @since IPv6
-     */
-    public static int selectRandomPort(RouterContext ctx) {
-        int minPort = Math.min(65535, Math.max(1, ctx.getProperty(PROP_MIN_PORT, MIN_RANDOM_PORT)));
-        int maxPort = Math.min(65535, Math.max(minPort, ctx.getProperty(PROP_MAX_PORT, MAX_RANDOM_PORT)));
-        return minPort + ctx.random().nextInt(1 + maxPort - minPort);
-    }
-
 
     /** call after startup() to get actual port or -1 on startup failure */
     public int getListenPort() { return _listenPort; }
